#include "config-athenz-provider-service.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::hosted::athenz::instanceproviderservice::config {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalAthenzProviderServiceType::CONFIG_DEF_MD5("f1af8b02b9bdde0ba0a4f084a1a36677");
const vespalib::string InternalAthenzProviderServiceType::CONFIG_DEF_NAME("athenz-provider-service");
const vespalib::string InternalAthenzProviderServiceType::CONFIG_DEF_NAMESPACE("vespa.hosted.athenz.instanceproviderservice.config");
const int64_t InternalAthenzProviderServiceType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.hosted.athenz.instanceproviderservice.config",
"domain string",
"serviceName string",
"secretName string",
"secretVersion int",
"sisSecretName string default=\"\"",
"sisSecretVersion int default=0",
"sisUrl string default = \"\"",
"caCertSecretName string",
"certDnsSuffix string default=\"\"",
"ztsUrl string default=\"\"",
"athenzCaTrustStore string default=\"\"",
"updatePeriodDays int default=1",
"tenantService string default=vespa.vespa.tenant",
};
const ::config::StringVector InternalAthenzProviderServiceType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalAthenzProviderServiceType::InternalAthenzProviderServiceType() noexcept
    : domain(),
      serviceName(),
      secretName(),
      secretVersion(0),
      sisSecretName(""),
      sisSecretVersion(0),
      sisUrl(""),
      caCertSecretName(),
      certDnsSuffix(""),
      ztsUrl(""),
      athenzCaTrustStore(""),
      updatePeriodDays(1),
      tenantService("vespa.vespa.tenant")
{
}

InternalAthenzProviderServiceType::InternalAthenzProviderServiceType(const InternalAthenzProviderServiceType & __rhs) noexcept = default;
InternalAthenzProviderServiceType & InternalAthenzProviderServiceType::operator =(const InternalAthenzProviderServiceType & __rhs) noexcept = default;
InternalAthenzProviderServiceType::InternalAthenzProviderServiceType(InternalAthenzProviderServiceType && __rhs) noexcept = default;
InternalAthenzProviderServiceType & InternalAthenzProviderServiceType::operator =(InternalAthenzProviderServiceType && __rhs) noexcept = default;
InternalAthenzProviderServiceType::~InternalAthenzProviderServiceType() = default; 
InternalAthenzProviderServiceType::InternalAthenzProviderServiceType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        domain = ConfigParser::parse<vespalib::string>("domain", __lines);
        ConfigParser::stripLinesForKey("domain", __remainingValuesToParse);
        serviceName = ConfigParser::parse<vespalib::string>("serviceName", __lines);
        ConfigParser::stripLinesForKey("serviceName", __remainingValuesToParse);
        secretName = ConfigParser::parse<vespalib::string>("secretName", __lines);
        ConfigParser::stripLinesForKey("secretName", __remainingValuesToParse);
        secretVersion = ConfigParser::parse<int32_t>("secretVersion", __lines);
        ConfigParser::stripLinesForKey("secretVersion", __remainingValuesToParse);
        sisSecretName = ConfigParser::parse<vespalib::string>("sisSecretName", __lines, "");
        ConfigParser::stripLinesForKey("sisSecretName", __remainingValuesToParse);
        sisSecretVersion = ConfigParser::parse<int32_t>("sisSecretVersion", __lines, 0);
        ConfigParser::stripLinesForKey("sisSecretVersion", __remainingValuesToParse);
        sisUrl = ConfigParser::parse<vespalib::string>("sisUrl", __lines, "");
        ConfigParser::stripLinesForKey("sisUrl", __remainingValuesToParse);
        caCertSecretName = ConfigParser::parse<vespalib::string>("caCertSecretName", __lines);
        ConfigParser::stripLinesForKey("caCertSecretName", __remainingValuesToParse);
        certDnsSuffix = ConfigParser::parse<vespalib::string>("certDnsSuffix", __lines, "");
        ConfigParser::stripLinesForKey("certDnsSuffix", __remainingValuesToParse);
        ztsUrl = ConfigParser::parse<vespalib::string>("ztsUrl", __lines, "");
        ConfigParser::stripLinesForKey("ztsUrl", __remainingValuesToParse);
        athenzCaTrustStore = ConfigParser::parse<vespalib::string>("athenzCaTrustStore", __lines, "");
        ConfigParser::stripLinesForKey("athenzCaTrustStore", __remainingValuesToParse);
        updatePeriodDays = ConfigParser::parse<int32_t>("updatePeriodDays", __lines, 1);
        ConfigParser::stripLinesForKey("updatePeriodDays", __remainingValuesToParse);
        tenantService = ConfigParser::parse<vespalib::string>("tenantService", __lines, "vespa.vespa.tenant");
        ConfigParser::stripLinesForKey("tenantService", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalAthenzProviderServiceType::
operator==(const InternalAthenzProviderServiceType& __rhs) const noexcept
{
    return (domain == __rhs.domain &&
            serviceName == __rhs.serviceName &&
            secretName == __rhs.secretName &&
            secretVersion == __rhs.secretVersion &&
            sisSecretName == __rhs.sisSecretName &&
            sisSecretVersion == __rhs.sisSecretVersion &&
            sisUrl == __rhs.sisUrl &&
            caCertSecretName == __rhs.caCertSecretName &&
            certDnsSuffix == __rhs.certDnsSuffix &&
            ztsUrl == __rhs.ztsUrl &&
            athenzCaTrustStore == __rhs.athenzCaTrustStore &&
            updatePeriodDays == __rhs.updatePeriodDays &&
            tenantService == __rhs.tenantService);
}

bool
InternalAthenzProviderServiceType::
operator!=(const InternalAthenzProviderServiceType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalAthenzProviderServiceType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("domain");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(domain));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("serviceName");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(serviceName));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("secretName");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(secretName));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("secretVersion");
        __c.setString("type", "int");
        __c.setLong("value", secretVersion);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("sisSecretName");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(sisSecretName));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("sisSecretVersion");
        __c.setString("type", "int");
        __c.setLong("value", sisSecretVersion);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("sisUrl");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(sisUrl));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("caCertSecretName");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(caCertSecretName));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("certDnsSuffix");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(certDnsSuffix));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("ztsUrl");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(ztsUrl));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("athenzCaTrustStore");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(athenzCaTrustStore));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("updatePeriodDays");
        __c.setString("type", "int");
        __c.setLong("value", updatePeriodDays);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("tenantService");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(tenantService));
    }
}

InternalAthenzProviderServiceType::InternalAthenzProviderServiceType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    domain = __inspector["domain"]["value"].asString().make_string();
    serviceName = __inspector["serviceName"]["value"].asString().make_string();
    secretName = __inspector["secretName"]["value"].asString().make_string();
    secretVersion = __inspector["secretVersion"]["value"].asLong();
    sisSecretName = __inspector["sisSecretName"]["value"].asString().make_string();
    sisSecretVersion = __inspector["sisSecretVersion"]["value"].asLong();
    sisUrl = __inspector["sisUrl"]["value"].asString().make_string();
    caCertSecretName = __inspector["caCertSecretName"]["value"].asString().make_string();
    certDnsSuffix = __inspector["certDnsSuffix"]["value"].asString().make_string();
    ztsUrl = __inspector["ztsUrl"]["value"].asString().make_string();
    athenzCaTrustStore = __inspector["athenzCaTrustStore"]["value"].asString().make_string();
    updatePeriodDays = __inspector["updatePeriodDays"]["value"].asLong();
    tenantService = __inspector["tenantService"]["value"].asString().make_string();
}

InternalAthenzProviderServiceType::InternalAthenzProviderServiceType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    domain = ::config::internal::ValueConverter<vespalib::string>()("domain", __inspector["domain"]);
    serviceName = ::config::internal::ValueConverter<vespalib::string>()("serviceName", __inspector["serviceName"]);
    secretName = ::config::internal::ValueConverter<vespalib::string>()("secretName", __inspector["secretName"]);
    secretVersion = ::config::internal::ValueConverter<int32_t>()("secretVersion", __inspector["secretVersion"]);
    sisSecretName = ::config::internal::ValueConverter<vespalib::string>()(__inspector["sisSecretName"], "");
    sisSecretVersion = ::config::internal::ValueConverter<int32_t>()(__inspector["sisSecretVersion"], 0);
    sisUrl = ::config::internal::ValueConverter<vespalib::string>()(__inspector["sisUrl"], "");
    caCertSecretName = ::config::internal::ValueConverter<vespalib::string>()("caCertSecretName", __inspector["caCertSecretName"]);
    certDnsSuffix = ::config::internal::ValueConverter<vespalib::string>()(__inspector["certDnsSuffix"], "");
    ztsUrl = ::config::internal::ValueConverter<vespalib::string>()(__inspector["ztsUrl"], "");
    athenzCaTrustStore = ::config::internal::ValueConverter<vespalib::string>()(__inspector["athenzCaTrustStore"], "");
    updatePeriodDays = ::config::internal::ValueConverter<int32_t>()(__inspector["updatePeriodDays"], 1);
    tenantService = ::config::internal::ValueConverter<vespalib::string>()(__inspector["tenantService"], "vespa.vespa.tenant");
}

} // namespace internal

} // namespace vespa::hosted::athenz::instanceproviderservice::config
