#include "config-stor-filestor.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::content {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalStorFilestorType::CONFIG_DEF_MD5("532a5bd97bc804f62b8c0791994247eb");
const vespalib::string InternalStorFilestorType::CONFIG_DEF_NAME("stor-filestor");
const vespalib::string InternalStorFilestorType::CONFIG_DEF_NAMESPACE("vespa.config.content");
const int64_t InternalStorFilestorType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.content",
"fail_disk_after_error_count int default=1 restart",
"disk_operation_timeout int default=0 restart",
"num_threads int default=8 restart",
"num_response_threads int default=2 restart",
"num_visitor_threads int default=16 restart",
"num_network_threads int default=2 restart",
"response_sequencer_type enum {LATENCY, THROUGHPUT, ADAPTIVE} default=ADAPTIVE restart",
"common_merge_chain_optimalization_minimum_size int default=64 restart",
"bucket_merge_chunk_size int default=16772216 restart",
"enable_merge_local_node_choose_docs_optimalization bool default=true restart",
"enable_multibit_split_optimalization bool default=true restart",
"use_async_message_handling_on_schedule bool default=false restart",
"resource_usage_reporter_noise_level double default=0.001",
"async_operation_throttler.type enum { UNLIMITED, DYNAMIC } default=DYNAMIC",
"async_operation_throttler.window_size_increment int default=20",
"async_operation_throttler.window_size_decrement_factor double default=1.2",
"async_operation_throttler.window_size_backoff double default=0.95",
"async_operation_throttler.min_window_size int default=20",
"async_operation_throttler.max_window_size int default=-1",
"async_operation_throttler.resize_rate double default=3.0",
"async_operation_throttler.throttle_individual_merge_feed_ops bool default=true",
"async_operation_throttler_type enum { UNLIMITED, DYNAMIC } default=DYNAMIC",
"async_operation_dynamic_throttling_window_increment int default=20 restart",
"use_per_document_throttled_delete_bucket bool default=false",
};
const ::config::StringVector InternalStorFilestorType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalStorFilestorType::ResponseSequencerType
InternalStorFilestorType::getResponseSequencerType(const vespalib::string& name)
{
    if (name == "LATENCY") {
        return ResponseSequencerType::LATENCY;
    } else if (name == "THROUGHPUT") {
        return ResponseSequencerType::THROUGHPUT;
    } else if (name == "ADAPTIVE") {
        return ResponseSequencerType::ADAPTIVE;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalStorFilestorType::getResponseSequencerTypeName(ResponseSequencerType t)
{
    switch (t) {
        case ResponseSequencerType::LATENCY: return "LATENCY";
        case ResponseSequencerType::THROUGHPUT: return "THROUGHPUT";
        case ResponseSequencerType::ADAPTIVE: return "ADAPTIVE";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalStorFilestorType::ResponseSequencerType InternalStorFilestorType::InternalResponseSequencerTypeConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalStorFilestorType::getResponseSequencerType(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalStorFilestorType::ResponseSequencerType InternalStorFilestorType::InternalResponseSequencerTypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalStorFilestorType::getResponseSequencerType(__inspector.asString().make_string());
}
InternalStorFilestorType::ResponseSequencerType InternalStorFilestorType::InternalResponseSequencerTypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector, ResponseSequencerType __eDefault) {
    if (__inspector.valid()) {
        return InternalStorFilestorType::getResponseSequencerType(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalStorFilestorType::AsyncOperationThrottler::Type
InternalStorFilestorType::AsyncOperationThrottler::getType(const vespalib::string& name)
{
    if (name == "UNLIMITED") {
        return Type::UNLIMITED;
    } else if (name == "DYNAMIC") {
        return Type::DYNAMIC;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalStorFilestorType::AsyncOperationThrottler::getTypeName(Type t)
{
    switch (t) {
        case Type::UNLIMITED: return "UNLIMITED";
        case Type::DYNAMIC: return "DYNAMIC";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalStorFilestorType::AsyncOperationThrottler::Type InternalStorFilestorType::AsyncOperationThrottler::InternalTypeConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalStorFilestorType::AsyncOperationThrottler::getType(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalStorFilestorType::AsyncOperationThrottler::Type InternalStorFilestorType::AsyncOperationThrottler::InternalTypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalStorFilestorType::AsyncOperationThrottler::getType(__inspector.asString().make_string());
}
InternalStorFilestorType::AsyncOperationThrottler::Type InternalStorFilestorType::AsyncOperationThrottler::InternalTypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Type __eDefault) {
    if (__inspector.valid()) {
        return InternalStorFilestorType::AsyncOperationThrottler::getType(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalStorFilestorType::AsyncOperationThrottler::AsyncOperationThrottler() noexcept
    : type(Type::DYNAMIC),
      windowSizeIncrement(20),
      windowSizeDecrementFactor(1.2),
      windowSizeBackoff(0.95),
      minWindowSize(20),
      maxWindowSize(-1),
      resizeRate(3.0),
      throttleIndividualMergeFeedOps(true)
{
}

InternalStorFilestorType::AsyncOperationThrottler::AsyncOperationThrottler(const AsyncOperationThrottler & __rhs) noexcept = default;
InternalStorFilestorType::AsyncOperationThrottler & InternalStorFilestorType::AsyncOperationThrottler::operator = (const AsyncOperationThrottler & __rhs) noexcept = default;
InternalStorFilestorType::AsyncOperationThrottler::AsyncOperationThrottler(AsyncOperationThrottler && __rhs) noexcept = default;
InternalStorFilestorType::AsyncOperationThrottler & InternalStorFilestorType::AsyncOperationThrottler::operator = (AsyncOperationThrottler && __rhs) noexcept = default;
InternalStorFilestorType::AsyncOperationThrottler::~AsyncOperationThrottler() = default; 
InternalStorFilestorType::AsyncOperationThrottler::AsyncOperationThrottler(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    type = getType(ConfigParser::parse<vespalib::string>("type", __lines, "DYNAMIC"));
    ConfigParser::stripLinesForKey("type", __remainingValuesToParse);
    windowSizeIncrement = ConfigParser::parse<int32_t>("window_size_increment", __lines, 20);
    ConfigParser::stripLinesForKey("window_size_increment", __remainingValuesToParse);
    windowSizeDecrementFactor = ConfigParser::parse<double>("window_size_decrement_factor", __lines, 1.2);
    ConfigParser::stripLinesForKey("window_size_decrement_factor", __remainingValuesToParse);
    windowSizeBackoff = ConfigParser::parse<double>("window_size_backoff", __lines, 0.95);
    ConfigParser::stripLinesForKey("window_size_backoff", __remainingValuesToParse);
    minWindowSize = ConfigParser::parse<int32_t>("min_window_size", __lines, 20);
    ConfigParser::stripLinesForKey("min_window_size", __remainingValuesToParse);
    maxWindowSize = ConfigParser::parse<int32_t>("max_window_size", __lines, -1);
    ConfigParser::stripLinesForKey("max_window_size", __remainingValuesToParse);
    resizeRate = ConfigParser::parse<double>("resize_rate", __lines, 3.0);
    ConfigParser::stripLinesForKey("resize_rate", __remainingValuesToParse);
    throttleIndividualMergeFeedOps = ConfigParser::parse<bool>("throttle_individual_merge_feed_ops", __lines, true);
    ConfigParser::stripLinesForKey("throttle_individual_merge_feed_ops", __remainingValuesToParse);
}

bool
InternalStorFilestorType::AsyncOperationThrottler::
operator==(const AsyncOperationThrottler& __rhs) const noexcept
{
    return (type == __rhs.type &&
            windowSizeIncrement == __rhs.windowSizeIncrement &&
            windowSizeDecrementFactor == __rhs.windowSizeDecrementFactor &&
            windowSizeBackoff == __rhs.windowSizeBackoff &&
            minWindowSize == __rhs.minWindowSize &&
            maxWindowSize == __rhs.maxWindowSize &&
            resizeRate == __rhs.resizeRate &&
            throttleIndividualMergeFeedOps == __rhs.throttleIndividualMergeFeedOps);
}

bool
InternalStorFilestorType::AsyncOperationThrottler::
operator!=(const AsyncOperationThrottler& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalStorFilestorType::AsyncOperationThrottler::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("type");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getTypeName(type)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("window_size_increment");
        __c.setString("type", "int");
        __c.setLong("value", windowSizeIncrement);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("window_size_decrement_factor");
        __c.setString("type", "double");
        __c.setDouble("value", windowSizeDecrementFactor);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("window_size_backoff");
        __c.setString("type", "double");
        __c.setDouble("value", windowSizeBackoff);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("min_window_size");
        __c.setString("type", "int");
        __c.setLong("value", minWindowSize);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("max_window_size");
        __c.setString("type", "int");
        __c.setLong("value", maxWindowSize);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("resize_rate");
        __c.setString("type", "double");
        __c.setDouble("value", resizeRate);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("throttle_individual_merge_feed_ops");
        __c.setString("type", "bool");
        __c.setBool("value", throttleIndividualMergeFeedOps);
    }
}

InternalStorFilestorType::AsyncOperationThrottler::AsyncOperationThrottler(const vespalib::slime::Inspector & __inspector)
{
    type = getType(__inspector["type"]["value"].asString().make_string());
    windowSizeIncrement = __inspector["window_size_increment"]["value"].asLong();
    windowSizeDecrementFactor = __inspector["window_size_decrement_factor"]["value"].asDouble();
    windowSizeBackoff = __inspector["window_size_backoff"]["value"].asDouble();
    minWindowSize = __inspector["min_window_size"]["value"].asLong();
    maxWindowSize = __inspector["max_window_size"]["value"].asLong();
    resizeRate = __inspector["resize_rate"]["value"].asDouble();
    throttleIndividualMergeFeedOps = __inspector["throttle_individual_merge_feed_ops"]["value"].asBool();
}

InternalStorFilestorType::AsyncOperationThrottler::AsyncOperationThrottler(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    type = InternalTypeConverter()(__inspector["type"], Type::DYNAMIC);
    windowSizeIncrement = ::config::internal::ValueConverter<int32_t>()(__inspector["window_size_increment"], 20);
    windowSizeDecrementFactor = ::config::internal::ValueConverter<double>()(__inspector["window_size_decrement_factor"], 1.2);
    windowSizeBackoff = ::config::internal::ValueConverter<double>()(__inspector["window_size_backoff"], 0.95);
    minWindowSize = ::config::internal::ValueConverter<int32_t>()(__inspector["min_window_size"], 20);
    maxWindowSize = ::config::internal::ValueConverter<int32_t>()(__inspector["max_window_size"], -1);
    resizeRate = ::config::internal::ValueConverter<double>()(__inspector["resize_rate"], 3.0);
    throttleIndividualMergeFeedOps = ::config::internal::ValueConverter<bool>()(__inspector["throttle_individual_merge_feed_ops"], true);
}

InternalStorFilestorType::AsyncOperationThrottlerType
InternalStorFilestorType::getAsyncOperationThrottlerType(const vespalib::string& name)
{
    if (name == "UNLIMITED") {
        return AsyncOperationThrottlerType::UNLIMITED;
    } else if (name == "DYNAMIC") {
        return AsyncOperationThrottlerType::DYNAMIC;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalStorFilestorType::getAsyncOperationThrottlerTypeName(AsyncOperationThrottlerType t)
{
    switch (t) {
        case AsyncOperationThrottlerType::UNLIMITED: return "UNLIMITED";
        case AsyncOperationThrottlerType::DYNAMIC: return "DYNAMIC";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalStorFilestorType::AsyncOperationThrottlerType InternalStorFilestorType::InternalAsyncOperationThrottlerTypeConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalStorFilestorType::getAsyncOperationThrottlerType(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalStorFilestorType::AsyncOperationThrottlerType InternalStorFilestorType::InternalAsyncOperationThrottlerTypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalStorFilestorType::getAsyncOperationThrottlerType(__inspector.asString().make_string());
}
InternalStorFilestorType::AsyncOperationThrottlerType InternalStorFilestorType::InternalAsyncOperationThrottlerTypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector, AsyncOperationThrottlerType __eDefault) {
    if (__inspector.valid()) {
        return InternalStorFilestorType::getAsyncOperationThrottlerType(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalStorFilestorType::InternalStorFilestorType() noexcept
    : failDiskAfterErrorCount(1),
      diskOperationTimeout(0),
      numThreads(8),
      numResponseThreads(2),
      numVisitorThreads(16),
      numNetworkThreads(2),
      responseSequencerType(ResponseSequencerType::ADAPTIVE),
      commonMergeChainOptimalizationMinimumSize(64),
      bucketMergeChunkSize(16772216),
      enableMergeLocalNodeChooseDocsOptimalization(true),
      enableMultibitSplitOptimalization(true),
      useAsyncMessageHandlingOnSchedule(false),
      resourceUsageReporterNoiseLevel(0.001),
      asyncOperationThrottler(),
      asyncOperationThrottlerType(AsyncOperationThrottlerType::DYNAMIC),
      asyncOperationDynamicThrottlingWindowIncrement(20),
      usePerDocumentThrottledDeleteBucket(false)
{
}

InternalStorFilestorType::InternalStorFilestorType(const InternalStorFilestorType & __rhs) noexcept = default;
InternalStorFilestorType & InternalStorFilestorType::operator =(const InternalStorFilestorType & __rhs) noexcept = default;
InternalStorFilestorType::InternalStorFilestorType(InternalStorFilestorType && __rhs) noexcept = default;
InternalStorFilestorType & InternalStorFilestorType::operator =(InternalStorFilestorType && __rhs) noexcept = default;
InternalStorFilestorType::~InternalStorFilestorType() = default; 
InternalStorFilestorType::InternalStorFilestorType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        failDiskAfterErrorCount = ConfigParser::parse<int32_t>("fail_disk_after_error_count", __lines, 1);
        ConfigParser::stripLinesForKey("fail_disk_after_error_count", __remainingValuesToParse);
        diskOperationTimeout = ConfigParser::parse<int32_t>("disk_operation_timeout", __lines, 0);
        ConfigParser::stripLinesForKey("disk_operation_timeout", __remainingValuesToParse);
        numThreads = ConfigParser::parse<int32_t>("num_threads", __lines, 8);
        ConfigParser::stripLinesForKey("num_threads", __remainingValuesToParse);
        numResponseThreads = ConfigParser::parse<int32_t>("num_response_threads", __lines, 2);
        ConfigParser::stripLinesForKey("num_response_threads", __remainingValuesToParse);
        numVisitorThreads = ConfigParser::parse<int32_t>("num_visitor_threads", __lines, 16);
        ConfigParser::stripLinesForKey("num_visitor_threads", __remainingValuesToParse);
        numNetworkThreads = ConfigParser::parse<int32_t>("num_network_threads", __lines, 2);
        ConfigParser::stripLinesForKey("num_network_threads", __remainingValuesToParse);
        responseSequencerType = getResponseSequencerType(ConfigParser::parse<vespalib::string>("response_sequencer_type", __lines, "ADAPTIVE"));
        ConfigParser::stripLinesForKey("response_sequencer_type", __remainingValuesToParse);
        commonMergeChainOptimalizationMinimumSize = ConfigParser::parse<int32_t>("common_merge_chain_optimalization_minimum_size", __lines, 64);
        ConfigParser::stripLinesForKey("common_merge_chain_optimalization_minimum_size", __remainingValuesToParse);
        bucketMergeChunkSize = ConfigParser::parse<int32_t>("bucket_merge_chunk_size", __lines, 16772216);
        ConfigParser::stripLinesForKey("bucket_merge_chunk_size", __remainingValuesToParse);
        enableMergeLocalNodeChooseDocsOptimalization = ConfigParser::parse<bool>("enable_merge_local_node_choose_docs_optimalization", __lines, true);
        ConfigParser::stripLinesForKey("enable_merge_local_node_choose_docs_optimalization", __remainingValuesToParse);
        enableMultibitSplitOptimalization = ConfigParser::parse<bool>("enable_multibit_split_optimalization", __lines, true);
        ConfigParser::stripLinesForKey("enable_multibit_split_optimalization", __remainingValuesToParse);
        useAsyncMessageHandlingOnSchedule = ConfigParser::parse<bool>("use_async_message_handling_on_schedule", __lines, false);
        ConfigParser::stripLinesForKey("use_async_message_handling_on_schedule", __remainingValuesToParse);
        resourceUsageReporterNoiseLevel = ConfigParser::parse<double>("resource_usage_reporter_noise_level", __lines, 0.001);
        ConfigParser::stripLinesForKey("resource_usage_reporter_noise_level", __remainingValuesToParse);
        asyncOperationThrottler = ConfigParser::parseStruct<AsyncOperationThrottler>("async_operation_throttler", __lines);
        ConfigParser::stripLinesForKey("async_operation_throttler", __remainingValuesToParse);
        asyncOperationThrottlerType = getAsyncOperationThrottlerType(ConfigParser::parse<vespalib::string>("async_operation_throttler_type", __lines, "DYNAMIC"));
        ConfigParser::stripLinesForKey("async_operation_throttler_type", __remainingValuesToParse);
        asyncOperationDynamicThrottlingWindowIncrement = ConfigParser::parse<int32_t>("async_operation_dynamic_throttling_window_increment", __lines, 20);
        ConfigParser::stripLinesForKey("async_operation_dynamic_throttling_window_increment", __remainingValuesToParse);
        usePerDocumentThrottledDeleteBucket = ConfigParser::parse<bool>("use_per_document_throttled_delete_bucket", __lines, false);
        ConfigParser::stripLinesForKey("use_per_document_throttled_delete_bucket", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalStorFilestorType::
operator==(const InternalStorFilestorType& __rhs) const noexcept
{
    return (failDiskAfterErrorCount == __rhs.failDiskAfterErrorCount &&
            diskOperationTimeout == __rhs.diskOperationTimeout &&
            numThreads == __rhs.numThreads &&
            numResponseThreads == __rhs.numResponseThreads &&
            numVisitorThreads == __rhs.numVisitorThreads &&
            numNetworkThreads == __rhs.numNetworkThreads &&
            responseSequencerType == __rhs.responseSequencerType &&
            commonMergeChainOptimalizationMinimumSize == __rhs.commonMergeChainOptimalizationMinimumSize &&
            bucketMergeChunkSize == __rhs.bucketMergeChunkSize &&
            enableMergeLocalNodeChooseDocsOptimalization == __rhs.enableMergeLocalNodeChooseDocsOptimalization &&
            enableMultibitSplitOptimalization == __rhs.enableMultibitSplitOptimalization &&
            useAsyncMessageHandlingOnSchedule == __rhs.useAsyncMessageHandlingOnSchedule &&
            resourceUsageReporterNoiseLevel == __rhs.resourceUsageReporterNoiseLevel &&
            asyncOperationThrottler == __rhs.asyncOperationThrottler &&
            asyncOperationThrottlerType == __rhs.asyncOperationThrottlerType &&
            asyncOperationDynamicThrottlingWindowIncrement == __rhs.asyncOperationDynamicThrottlingWindowIncrement &&
            usePerDocumentThrottledDeleteBucket == __rhs.usePerDocumentThrottledDeleteBucket);
}

bool
InternalStorFilestorType::
operator!=(const InternalStorFilestorType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalStorFilestorType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("fail_disk_after_error_count");
        __c.setString("type", "int");
        __c.setLong("value", failDiskAfterErrorCount);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("disk_operation_timeout");
        __c.setString("type", "int");
        __c.setLong("value", diskOperationTimeout);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("num_threads");
        __c.setString("type", "int");
        __c.setLong("value", numThreads);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("num_response_threads");
        __c.setString("type", "int");
        __c.setLong("value", numResponseThreads);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("num_visitor_threads");
        __c.setString("type", "int");
        __c.setLong("value", numVisitorThreads);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("num_network_threads");
        __c.setString("type", "int");
        __c.setLong("value", numNetworkThreads);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("response_sequencer_type");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getResponseSequencerTypeName(responseSequencerType)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("common_merge_chain_optimalization_minimum_size");
        __c.setString("type", "int");
        __c.setLong("value", commonMergeChainOptimalizationMinimumSize);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("bucket_merge_chunk_size");
        __c.setString("type", "int");
        __c.setLong("value", bucketMergeChunkSize);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("enable_merge_local_node_choose_docs_optimalization");
        __c.setString("type", "bool");
        __c.setBool("value", enableMergeLocalNodeChooseDocsOptimalization);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("enable_multibit_split_optimalization");
        __c.setString("type", "bool");
        __c.setBool("value", enableMultibitSplitOptimalization);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("use_async_message_handling_on_schedule");
        __c.setString("type", "bool");
        __c.setBool("value", useAsyncMessageHandlingOnSchedule);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("resource_usage_reporter_noise_level");
        __c.setString("type", "double");
        __c.setDouble("value", resourceUsageReporterNoiseLevel);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("async_operation_throttler");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        asyncOperationThrottler.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("async_operation_throttler_type");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getAsyncOperationThrottlerTypeName(asyncOperationThrottlerType)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("async_operation_dynamic_throttling_window_increment");
        __c.setString("type", "int");
        __c.setLong("value", asyncOperationDynamicThrottlingWindowIncrement);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("use_per_document_throttled_delete_bucket");
        __c.setString("type", "bool");
        __c.setBool("value", usePerDocumentThrottledDeleteBucket);
    }
}

InternalStorFilestorType::InternalStorFilestorType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    failDiskAfterErrorCount = __inspector["fail_disk_after_error_count"]["value"].asLong();
    diskOperationTimeout = __inspector["disk_operation_timeout"]["value"].asLong();
    numThreads = __inspector["num_threads"]["value"].asLong();
    numResponseThreads = __inspector["num_response_threads"]["value"].asLong();
    numVisitorThreads = __inspector["num_visitor_threads"]["value"].asLong();
    numNetworkThreads = __inspector["num_network_threads"]["value"].asLong();
    responseSequencerType = getResponseSequencerType(__inspector["response_sequencer_type"]["value"].asString().make_string());
    commonMergeChainOptimalizationMinimumSize = __inspector["common_merge_chain_optimalization_minimum_size"]["value"].asLong();
    bucketMergeChunkSize = __inspector["bucket_merge_chunk_size"]["value"].asLong();
    enableMergeLocalNodeChooseDocsOptimalization = __inspector["enable_merge_local_node_choose_docs_optimalization"]["value"].asBool();
    enableMultibitSplitOptimalization = __inspector["enable_multibit_split_optimalization"]["value"].asBool();
    useAsyncMessageHandlingOnSchedule = __inspector["use_async_message_handling_on_schedule"]["value"].asBool();
    resourceUsageReporterNoiseLevel = __inspector["resource_usage_reporter_noise_level"]["value"].asDouble();
    asyncOperationThrottler = AsyncOperationThrottler(__inspector["async_operation_throttler"]["value"]);
    asyncOperationThrottlerType = getAsyncOperationThrottlerType(__inspector["async_operation_throttler_type"]["value"].asString().make_string());
    asyncOperationDynamicThrottlingWindowIncrement = __inspector["async_operation_dynamic_throttling_window_increment"]["value"].asLong();
    usePerDocumentThrottledDeleteBucket = __inspector["use_per_document_throttled_delete_bucket"]["value"].asBool();
}

InternalStorFilestorType::InternalStorFilestorType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    failDiskAfterErrorCount = ::config::internal::ValueConverter<int32_t>()(__inspector["fail_disk_after_error_count"], 1);
    diskOperationTimeout = ::config::internal::ValueConverter<int32_t>()(__inspector["disk_operation_timeout"], 0);
    numThreads = ::config::internal::ValueConverter<int32_t>()(__inspector["num_threads"], 8);
    numResponseThreads = ::config::internal::ValueConverter<int32_t>()(__inspector["num_response_threads"], 2);
    numVisitorThreads = ::config::internal::ValueConverter<int32_t>()(__inspector["num_visitor_threads"], 16);
    numNetworkThreads = ::config::internal::ValueConverter<int32_t>()(__inspector["num_network_threads"], 2);
    responseSequencerType = InternalResponseSequencerTypeConverter()(__inspector["response_sequencer_type"], ResponseSequencerType::ADAPTIVE);
    commonMergeChainOptimalizationMinimumSize = ::config::internal::ValueConverter<int32_t>()(__inspector["common_merge_chain_optimalization_minimum_size"], 64);
    bucketMergeChunkSize = ::config::internal::ValueConverter<int32_t>()(__inspector["bucket_merge_chunk_size"], 16772216);
    enableMergeLocalNodeChooseDocsOptimalization = ::config::internal::ValueConverter<bool>()(__inspector["enable_merge_local_node_choose_docs_optimalization"], true);
    enableMultibitSplitOptimalization = ::config::internal::ValueConverter<bool>()(__inspector["enable_multibit_split_optimalization"], true);
    useAsyncMessageHandlingOnSchedule = ::config::internal::ValueConverter<bool>()(__inspector["use_async_message_handling_on_schedule"], false);
    resourceUsageReporterNoiseLevel = ::config::internal::ValueConverter<double>()(__inspector["resource_usage_reporter_noise_level"], 0.001);
    asyncOperationThrottler = ::config::internal::ValueConverter<AsyncOperationThrottler>()(__inspector["async_operation_throttler"]);
    asyncOperationThrottlerType = InternalAsyncOperationThrottlerTypeConverter()(__inspector["async_operation_throttler_type"], AsyncOperationThrottlerType::DYNAMIC);
    asyncOperationDynamicThrottlingWindowIncrement = ::config::internal::ValueConverter<int32_t>()(__inspector["async_operation_dynamic_throttling_window_increment"], 20);
    usePerDocumentThrottledDeleteBucket = ::config::internal::ValueConverter<bool>()(__inspector["use_per_document_throttled_delete_bucket"], false);
}

} // namespace internal

} // namespace vespa::config::content
