/**
 * @class vespa::config::content::StorFilestor
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 */
#ifndef CLOUD_CONFIG_VESPA_CONFIG_CONTENT_STORFILESTOR_H
#define CLOUD_CONFIG_VESPA_CONFIG_CONTENT_STORFILESTOR_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace vespa::config::content {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalStorFilestorType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalStorFilestorType> UP;
    enum class ResponseSequencerType { LATENCY, THROUGHPUT, ADAPTIVE };
    typedef std::vector<ResponseSequencerType> ResponseSequencerTypeVector;
    typedef std::map<vespalib::string, ResponseSequencerType> ResponseSequencerTypeMap;
    static ResponseSequencerType getResponseSequencerType(const vespalib::string&);
    static vespalib::string getResponseSequencerTypeName(ResponseSequencerType e);

    struct InternalResponseSequencerTypeConverter {
        ResponseSequencerType operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
        ResponseSequencerType operator()(const ::vespalib::slime::Inspector & __inspector);
        ResponseSequencerType operator()(const ::vespalib::slime::Inspector & __inspector, ResponseSequencerType __eDefault);
    };
    class AsyncOperationThrottler {
    public:
        enum class Type { UNLIMITED, DYNAMIC };
        typedef std::vector<Type> TypeVector;
        typedef std::map<vespalib::string, Type> TypeMap;
        static Type getType(const vespalib::string&);
        static vespalib::string getTypeName(Type e);

        struct InternalTypeConverter {
            Type operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
            Type operator()(const ::vespalib::slime::Inspector & __inspector);
            Type operator()(const ::vespalib::slime::Inspector & __inspector, Type __eDefault);
        };
        AsyncOperationThrottler(const ::config::StringVector & __lines);
        AsyncOperationThrottler(const vespalib::slime::Inspector & __inspector);
        AsyncOperationThrottler(const ::config::ConfigPayload & __payload);
        AsyncOperationThrottler()  noexcept;
        AsyncOperationThrottler(const AsyncOperationThrottler & __rhs) noexcept;
        AsyncOperationThrottler & operator = (const AsyncOperationThrottler & __rhs) noexcept;
        AsyncOperationThrottler(AsyncOperationThrottler && __rhs) noexcept;
        AsyncOperationThrottler & operator = (AsyncOperationThrottler && __rhs) noexcept;
        ~AsyncOperationThrottler();

        bool operator==(const AsyncOperationThrottler& __rhs) const noexcept;
        bool operator!=(const AsyncOperationThrottler& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /**
         * Specify throttling used for async persistence operations. This
         * throttling takes place
         * before operations are dispatched to Proton and serves as a limiter
         * for how many
         * operations may be in flight in Proton's internal queues.
         *  
         * - UNLIMITED is, as it says on the tin, unlimited. Offers no actual
         * throttling, but
         * has near zero overhead and never blocks.
         * - DYNAMIC uses DynamicThrottlePolicy under the hood and will block if
         * the window
         * is full (if a blocking throttler API call is invoked).
         *  
         */
        Type type; // Default: DYNAMIC

        /**
         * Internal throttler tuning parameters that only apply when type ==
         * DYNAMIC:
         */
        int32_t windowSizeIncrement; // Default: 20
        double windowSizeDecrementFactor; // Default: 1.2
        double windowSizeBackoff; // Default: 0.95
        int32_t minWindowSize; // Default: 20
        int32_t maxWindowSize; // Default: -1
        double resizeRate; // Default: 3.0

        /**
         * If true, each put/remove contained within a merge is individually
         * throttled as if it
         * were a put/remove from a client. If false, merges are throttled at a
         * persistence thread
         * level, i.e. per ApplyBucketDiff message, regardless of how many
         * document operations
         * are contained within.
         */
        bool throttleIndividualMergeFeedOps; // Default: true
    };
    typedef std::vector<AsyncOperationThrottler> AsyncOperationThrottlerVector;

    typedef std::map<vespalib::string, AsyncOperationThrottler> AsyncOperationThrottlerMap;

    enum class AsyncOperationThrottlerType { UNLIMITED, DYNAMIC };
    typedef std::vector<AsyncOperationThrottlerType> AsyncOperationThrottlerTypeVector;
    typedef std::map<vespalib::string, AsyncOperationThrottlerType> AsyncOperationThrottlerTypeMap;
    static AsyncOperationThrottlerType getAsyncOperationThrottlerType(const vespalib::string&);
    static vespalib::string getAsyncOperationThrottlerTypeName(AsyncOperationThrottlerType e);

    struct InternalAsyncOperationThrottlerTypeConverter {
        AsyncOperationThrottlerType operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
        AsyncOperationThrottlerType operator()(const ::vespalib::slime::Inspector & __inspector);
        AsyncOperationThrottlerType operator()(const ::vespalib::slime::Inspector & __inspector, AsyncOperationThrottlerType __eDefault);
    };
    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalStorFilestorType(const ::config::ConfigValue & __value);
    InternalStorFilestorType(const ::config::ConfigDataBuffer & __value);
    InternalStorFilestorType(const ::config::ConfigPayload & __payload);
    InternalStorFilestorType()  noexcept;
    InternalStorFilestorType(const InternalStorFilestorType & __rhs) noexcept;
    InternalStorFilestorType & operator = (const InternalStorFilestorType & __rhs) noexcept;
    InternalStorFilestorType(InternalStorFilestorType && __rhs) noexcept;
    InternalStorFilestorType & operator = (InternalStorFilestorType && __rhs) noexcept;
    ~InternalStorFilestorType();

    bool operator==(const InternalStorFilestorType& __rhs) const noexcept;
    bool operator!=(const InternalStorFilestorType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;


    // DETECT FAILURE PARAMETERS

    /** Deprecated and unused - will soon go away */
    int32_t failDiskAfterErrorCount; // Default: 1

    /** Deprecated and unused - will soon go away */
    int32_t diskOperationTimeout; // Default: 0

    // PERFORMANCE PARAMETERS

    /** Number of threads to use for each mountpoint. */
    int32_t numThreads; // Default: 8

    /**
     * Number of threads for response processing and delivery
     * 0 will give legacy sync behavior.
     * Negative number will choose a good number based on # cores.
     */
    int32_t numResponseThreads; // Default: 2

    /**
     * Number of handler objects that might be created by visitor threads
     * This is a temporary setting that only skilled vespa developers should
     * modify
     * This must be kept in line with stor-visitor:visitorthreads
     */
    int32_t numVisitorThreads; // Default: 16

    /**
     * Number of handler objects that might be created by network threads
     * This is a temporary setting that only skilled vespa developers should
     * modify
     * This must be kept in line with stor-communicationmanager:rpc.num_network_-
     * threads
     */
    int32_t numNetworkThreads; // Default: 2

    /** Type of sequenced thread executor use for persistence replies. */
    ResponseSequencerType responseSequencerType; // Default: ADAPTIVE

    /**
     * When merging, if we find more than this number of documents that exist on
     * all
     * of the same copies, send a separate apply bucket diff with these entries
     * to an optimized merge chain that guarantuees minimum data transfer.
     */
    int32_t commonMergeChainOptimalizationMinimumSize; // Default: 64

    /**
     * Chunksize to use while merging buckets between nodes.
     *  
     * Should follow stor-distributormanager:splitsize (16MB).
     */
    int32_t bucketMergeChunkSize; // Default: 16772216

    /**
     * When merging, it is possible to send more metadata than needed in order
     * to
     * let local nodes in merge decide which entries fits best to add this time
     * based on disk location. Toggle this option on to use it. Note that memory
     * consumption might increase in a 4.1 to 4.2 upgrade due to this, as 4.1
     * dont support to only fill in part of the metadata provided and will
     * always
     * fill all.
     * NB unused and will be removed shortly.
     */
    bool enableMergeLocalNodeChooseDocsOptimalization; // Default: true

    /**
     * Whether or not to enable the multibit split optimalization. This is
     * useful
     * if splitting is expensive, but listing document identifiers is fairly
     * cheap.
     * This is true for memfile persistence layer, but not for vespa search.
     */
    bool enableMultibitSplitOptimalization; // Default: true

    /**
     * Whether or not to use async message handling when scheduling storage
     * messages from FileStorManager.
     *  
     * When turned on, the calling thread (e.g. FNET network thread when using
     * Storage API RPC)
     * gets the next async message to handle (if any) as part of scheduling a
     * storage message.
     * This async message is then handled by the calling thread immediately,
     * instead of going via a persistence thread.
     */
    bool useAsyncMessageHandlingOnSchedule; // Default: false

    /**
     * The noise level used when deciding whether a resource usage sample should
     * be reported to the cluster controller.
     *  
     * If one of the resource categories (e.g. disk or memory) has a usage delta
     * that is larger than the noise level,
     * the entire resource usage sample is immediately reported to the cluster
     * controller (via host info).
     * This config can be live updated (doesn't require restart).
     */
    double resourceUsageReporterNoiseLevel; // Default: 0.001
    AsyncOperationThrottler asyncOperationThrottler;

    /**
     * Specify throttling used for async persistence operations. This throttling
     * takes place
     * before operations are dispatched to Proton and serves as a limiter for
     * how many
     * operations may be in flight in Proton's internal queues.
     *  
     * - UNLIMITED is, as it says on the tin, unlimited. Offers no actual
     * throttling, but
     * has near zero overhead and never blocks.
     * - DYNAMIC uses DynamicThrottlePolicy under the hood and will block if the
     * window
     * is full (if a blocking throttler API call is invoked).
     *  
     * TODO deprecate in favor of the async_operation_throttler struct instead.
     */
    AsyncOperationThrottlerType asyncOperationThrottlerType; // Default: DYNAMIC

    /**
     * Specifies the extent the throttling window is increased by when the async
     * throttle
     * policy has decided that more concurrent operations are desirable. Also
     * affects the
     * _minimum_ size of the throttling window; its size is implicitly set to
     * max(this config
     * value, number of threads).
     *  
     * Only applies if async_operation_throttler_type == DYNAMIC.
     * DEPRECATED! use the async_operation_throttler struct instead
     */
    int32_t asyncOperationDynamicThrottlingWindowIncrement; // Default: 20

    /**
     * If set, DeleteBucket operations are internally expanded to an
     * individually persistence-
     * throttled remove per document stored in the bucket. This makes the cost
     * model of
     * executing a DeleteBucket symmetrical with feeding the documents to the
     * bucket in the
     * first place.
     *  
     * This is a live config.
     */
    bool usePerDocumentThrottledDeleteBucket; // Default: false
};

} // namespace internal

typedef internal::InternalStorFilestorType StorFilestorConfigBuilder;
typedef const internal::InternalStorFilestorType StorFilestorConfig;

} // namespace vespa::config::content
#endif // VESPA_config_VESPA_CONFIG_CONTENT_STORFILESTOR_H
