#include "config-stor-bucketdb.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::content::core {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalStorBucketdbType::CONFIG_DEF_MD5("3a9a7fae16561ec23b8df819e4a3a46e");
const vespalib::string InternalStorBucketdbType::CONFIG_DEF_NAME("stor-bucketdb");
const vespalib::string InternalStorBucketdbType::CONFIG_DEF_NAMESPACE("vespa.config.content.core");
const int64_t InternalStorBucketdbType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.content.core",
"bucketinfobatchsize int default=128 restart",
"chunklevel int default=1 restart",
};
const ::config::StringVector InternalStorBucketdbType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalStorBucketdbType::InternalStorBucketdbType() noexcept
    : bucketinfobatchsize(128),
      chunklevel(1)
{
}

InternalStorBucketdbType::InternalStorBucketdbType(const InternalStorBucketdbType & __rhs) noexcept = default;
InternalStorBucketdbType & InternalStorBucketdbType::operator =(const InternalStorBucketdbType & __rhs) noexcept = default;
InternalStorBucketdbType::InternalStorBucketdbType(InternalStorBucketdbType && __rhs) noexcept = default;
InternalStorBucketdbType & InternalStorBucketdbType::operator =(InternalStorBucketdbType && __rhs) noexcept = default;
InternalStorBucketdbType::~InternalStorBucketdbType() = default; 
InternalStorBucketdbType::InternalStorBucketdbType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        bucketinfobatchsize = ConfigParser::parse<int32_t>("bucketinfobatchsize", __lines, 128);
        ConfigParser::stripLinesForKey("bucketinfobatchsize", __remainingValuesToParse);
        chunklevel = ConfigParser::parse<int32_t>("chunklevel", __lines, 1);
        ConfigParser::stripLinesForKey("chunklevel", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalStorBucketdbType::
operator==(const InternalStorBucketdbType& __rhs) const noexcept
{
    return (bucketinfobatchsize == __rhs.bucketinfobatchsize &&
            chunklevel == __rhs.chunklevel);
}

bool
InternalStorBucketdbType::
operator!=(const InternalStorBucketdbType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalStorBucketdbType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("bucketinfobatchsize");
        __c.setString("type", "int");
        __c.setLong("value", bucketinfobatchsize);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("chunklevel");
        __c.setString("type", "int");
        __c.setLong("value", chunklevel);
    }
}

InternalStorBucketdbType::InternalStorBucketdbType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    bucketinfobatchsize = __inspector["bucketinfobatchsize"]["value"].asLong();
    chunklevel = __inspector["chunklevel"]["value"].asLong();
}

InternalStorBucketdbType::InternalStorBucketdbType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    bucketinfobatchsize = ::config::internal::ValueConverter<int32_t>()(__inspector["bucketinfobatchsize"], 128);
    chunklevel = ::config::internal::ValueConverter<int32_t>()(__inspector["chunklevel"], 1);
}

} // namespace internal

} // namespace vespa::config::content::core
