#include "config-stor-bouncer.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::content::core {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalStorBouncerType::CONFIG_DEF_MD5("5b5b77d4c31a41c1d7de70d3fdb7f8ab");
const vespalib::string InternalStorBouncerType::CONFIG_DEF_NAME("stor-bouncer");
const vespalib::string InternalStorBouncerType::CONFIG_DEF_NAMESPACE("vespa.config.content.core");
const int64_t InternalStorBouncerType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.content.core",
"stop_external_load_when_cluster_down bool default=true",
"stop_all_load_when_nodestate_not_in string default=\"uri\"",
"use_wanted_state_if_possible bool default=true",
"max_clock_skew_seconds int default=5",
"feed_rejection_priority_threshold int default=-1",
};
const ::config::StringVector InternalStorBouncerType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalStorBouncerType::InternalStorBouncerType() noexcept
    : stopExternalLoadWhenClusterDown(true),
      stopAllLoadWhenNodestateNotIn("uri"),
      useWantedStateIfPossible(true),
      maxClockSkewSeconds(5),
      feedRejectionPriorityThreshold(-1)
{
}

InternalStorBouncerType::InternalStorBouncerType(const InternalStorBouncerType & __rhs) noexcept = default;
InternalStorBouncerType & InternalStorBouncerType::operator =(const InternalStorBouncerType & __rhs) noexcept = default;
InternalStorBouncerType::InternalStorBouncerType(InternalStorBouncerType && __rhs) noexcept = default;
InternalStorBouncerType & InternalStorBouncerType::operator =(InternalStorBouncerType && __rhs) noexcept = default;
InternalStorBouncerType::~InternalStorBouncerType() = default; 
InternalStorBouncerType::InternalStorBouncerType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        stopExternalLoadWhenClusterDown = ConfigParser::parse<bool>("stop_external_load_when_cluster_down", __lines, true);
        ConfigParser::stripLinesForKey("stop_external_load_when_cluster_down", __remainingValuesToParse);
        stopAllLoadWhenNodestateNotIn = ConfigParser::parse<vespalib::string>("stop_all_load_when_nodestate_not_in", __lines, "uri");
        ConfigParser::stripLinesForKey("stop_all_load_when_nodestate_not_in", __remainingValuesToParse);
        useWantedStateIfPossible = ConfigParser::parse<bool>("use_wanted_state_if_possible", __lines, true);
        ConfigParser::stripLinesForKey("use_wanted_state_if_possible", __remainingValuesToParse);
        maxClockSkewSeconds = ConfigParser::parse<int32_t>("max_clock_skew_seconds", __lines, 5);
        ConfigParser::stripLinesForKey("max_clock_skew_seconds", __remainingValuesToParse);
        feedRejectionPriorityThreshold = ConfigParser::parse<int32_t>("feed_rejection_priority_threshold", __lines, -1);
        ConfigParser::stripLinesForKey("feed_rejection_priority_threshold", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalStorBouncerType::operator==(const InternalStorBouncerType& __rhs) const noexcept
{
    return (stopExternalLoadWhenClusterDown == __rhs.stopExternalLoadWhenClusterDown &&
            stopAllLoadWhenNodestateNotIn == __rhs.stopAllLoadWhenNodestateNotIn &&
            useWantedStateIfPossible == __rhs.useWantedStateIfPossible &&
            maxClockSkewSeconds == __rhs.maxClockSkewSeconds &&
            feedRejectionPriorityThreshold == __rhs.feedRejectionPriorityThreshold);
}

bool
InternalStorBouncerType::operator!=(const InternalStorBouncerType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalStorBouncerType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("stop_external_load_when_cluster_down");
        __c.setString("type", "bool");
        __c.setBool("value", stopExternalLoadWhenClusterDown);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("stop_all_load_when_nodestate_not_in");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(stopAllLoadWhenNodestateNotIn));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("use_wanted_state_if_possible");
        __c.setString("type", "bool");
        __c.setBool("value", useWantedStateIfPossible);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("max_clock_skew_seconds");
        __c.setString("type", "int");
        __c.setLong("value", maxClockSkewSeconds);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("feed_rejection_priority_threshold");
        __c.setString("type", "int");
        __c.setLong("value", feedRejectionPriorityThreshold);
    }
}

InternalStorBouncerType::InternalStorBouncerType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    stopExternalLoadWhenClusterDown = __inspector["stop_external_load_when_cluster_down"]["value"].asBool();
    stopAllLoadWhenNodestateNotIn = __inspector["stop_all_load_when_nodestate_not_in"]["value"].asString().make_string();
    useWantedStateIfPossible = __inspector["use_wanted_state_if_possible"]["value"].asBool();
    maxClockSkewSeconds = __inspector["max_clock_skew_seconds"]["value"].asLong();
    feedRejectionPriorityThreshold = __inspector["feed_rejection_priority_threshold"]["value"].asLong();
}

InternalStorBouncerType::InternalStorBouncerType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    stopExternalLoadWhenClusterDown = ::config::internal::ValueConverter<bool>()(__inspector["stop_external_load_when_cluster_down"], true);
    stopAllLoadWhenNodestateNotIn = ::config::internal::ValueConverter<vespalib::string>()(__inspector["stop_all_load_when_nodestate_not_in"], "uri");
    useWantedStateIfPossible = ::config::internal::ValueConverter<bool>()(__inspector["use_wanted_state_if_possible"], true);
    maxClockSkewSeconds = ::config::internal::ValueConverter<int32_t>()(__inspector["max_clock_skew_seconds"], 5);
    feedRejectionPriorityThreshold = ::config::internal::ValueConverter<int32_t>()(__inspector["feed_rejection_priority_threshold"], -1);
}

} // namespace internal

} // namespace vespa::config::content::core
