/**
 * @class vespa::config::content::core::StorBouncer
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 */
#ifndef CLOUD_CONFIG_VESPA_CONFIG_CONTENT_CORE_STORBOUNCER_H
#define CLOUD_CONFIG_VESPA_CONFIG_CONTENT_CORE_STORBOUNCER_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace vespa::config::content::core {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalStorBouncerType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalStorBouncerType> UP;
    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalStorBouncerType(const ::config::ConfigValue & __value);
    InternalStorBouncerType(const ::config::ConfigDataBuffer & __value);
    InternalStorBouncerType(const ::config::ConfigPayload & __payload);
    InternalStorBouncerType()  noexcept;
    InternalStorBouncerType(const InternalStorBouncerType & __rhs) noexcept;
    InternalStorBouncerType & operator = (const InternalStorBouncerType & __rhs) noexcept;
    InternalStorBouncerType(InternalStorBouncerType && __rhs) noexcept;
    InternalStorBouncerType & operator = (InternalStorBouncerType && __rhs) noexcept;
    ~InternalStorBouncerType();

    bool operator==(const InternalStorBouncerType& __rhs) const noexcept;
    bool operator!=(const InternalStorBouncerType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;


    /**
     * Whether or not the bouncer should stop external load from
     * entering node when the cluster state is down.
     */
    bool stopExternalLoadWhenClusterDown; // Default: true

    /**
     * Sets what node states the node will allow incoming commands
     * in.
     */
    vespalib::string stopAllLoadWhenNodestateNotIn; // Default: "uri"

    /**
     * Sets whether to just use (self) reported node state or to use wanted
     * state
     * if wanted state is worse than the current reported state.
     */
    bool useWantedStateIfPossible; // Default: true

    /**
     * The maximum clock skew allowed in the system. Any messages received
     * that have a timestamp longer in the future than this will be failed.
     */
    int32_t maxClockSkewSeconds; // Default: 5

    /**
     * If this config value is != -1, the node will reject any external feed
     * operations with a priority lower than that specified here. Note that
     * since
     * we map priorities in such a way that 0 is the _highest_ priority and 255
     * the
     * _lowest_ priority, for two operations A and B, if B has a lower priority
     * than A it will have a higher priority _integer_ value.
     *  
     * Only mutating external feed operations will be blocked. Read-only
     * operations
     * and internal operations are always let through.
     *  
     * Default is -1 (i.e. rejection is disabled and load is allowed through)
     */
    int32_t feedRejectionPriorityThreshold; // Default: -1
};

} // namespace internal

typedef internal::InternalStorBouncerType StorBouncerConfigBuilder;
typedef const internal::InternalStorBouncerType StorBouncerConfig;

} // namespace vespa::config::content::core
#endif // VESPA_config_VESPA_CONFIG_CONTENT_CORE_STORBOUNCER_H
