#include "config-stor-distributormanager.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::content::core {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalStorDistributormanagerType::CONFIG_DEF_MD5("f2a73c17df031dc26dafed92ed8e0370");
const vespalib::string InternalStorDistributormanagerType::CONFIG_DEF_NAME("stor-distributormanager");
const vespalib::string InternalStorDistributormanagerType::CONFIG_DEF_NAMESPACE("vespa.config.content.core");
const int64_t InternalStorDistributormanagerType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.content.core",
"maxpendingidealstateoperations int default=100",
"splitsize int default=16772216",
"splitcount int default=1024",
"joinsize int default=16000000",
"joincount int default=512",
"minsplitcount int default=16",
"garbagecollection.selectiontoremove string default=\"\"",
"garbagecollection.interval int default=0",
"inlinebucketsplitting bool default=true",
"blockedstatecheckers[] string restart",
"enable_revert bool default=true",
"maximum_nodes_per_merge int default=16",
"start_distributor_thread bool default=true restart",
"ticks_before_wait int default=10",
"ticks_wait_time_ms int default=1",
"max_process_time_ms int default=5000",
"priority_merge_move_to_ideal_node int default=165",
"priority_merge_out_of_sync_copies int default=120",
"priority_merge_too_few_copies int default=120",
"priority_merge_global_buckets int default=115",
"priority_activate_no_existing_active int default=100",
"priority_activate_with_existing_active int default=100",
"priority_delete_bucket_copy int default=120",
"priority_join_buckets int default=155",
"priority_split_distribution_bits int default=200",
"priority_split_large_bucket int default=175",
"priority_split_inconsistent_bucket int default=110",
"priority_garbage_collection int default=200",
"enable_join_for_sibling_less_buckets bool default=false",
"enable_inconsistent_join bool default=false",
"enable_host_info_reporting bool default=true",
"minimum_replica_counting_mode enum { TRUSTED, ANY } default=TRUSTED",
"disable_bucket_activation bool default=false",
"max_cluster_clock_skew_sec int default=1",
"sequence_mutating_operations bool default=true",
"inhibit_merge_sending_on_busy_node_duration_sec int default=1",
"allow_stale_reads_during_cluster_state_transitions bool default=false",
"simulated_db_pruning_latency_msec int default=0",
"simulated_db_merging_latency_msec int default=0",
"use_btree_database bool default=true restart",
"restart_with_fast_update_path_if_all_get_timestamps_are_consistent bool default=true",
"merge_operations_disabled bool default=false",
"use_weak_internal_read_consistency_for_client_gets bool default=false",
"enable_metadata_only_fetch_phase_for_inconsistent_updates bool default=true",
"max_consecutively_inhibited_maintenance_ticks int default=20",
"prioritize_global_bucket_merges bool default=true",
"max_activation_inhibited_out_of_sync_groups int default=0",
"num_distributor_stripes int default=0 restart",
"implicitly_clear_bucket_priority_on_schedule bool default=true",
"use_unordered_merge_chaining bool default=true",
"inhibit_default_merges_when_global_merges_pending bool default=true",
"enable_two_phase_garbage_collection bool default=true",
"enable_condition_probing bool default=true",
"enable_operation_cancellation bool default=false",
};
const ::config::StringVector InternalStorDistributormanagerType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalStorDistributormanagerType::Garbagecollection::Garbagecollection() noexcept
    : selectiontoremove(""),
      interval(0)
{
}

InternalStorDistributormanagerType::Garbagecollection::Garbagecollection(const Garbagecollection & __rhs) noexcept = default;
InternalStorDistributormanagerType::Garbagecollection & InternalStorDistributormanagerType::Garbagecollection::operator = (const Garbagecollection & __rhs) noexcept = default;
InternalStorDistributormanagerType::Garbagecollection::Garbagecollection(Garbagecollection && __rhs) noexcept = default;
InternalStorDistributormanagerType::Garbagecollection & InternalStorDistributormanagerType::Garbagecollection::operator = (Garbagecollection && __rhs) noexcept = default;
InternalStorDistributormanagerType::Garbagecollection::~Garbagecollection() = default; 
InternalStorDistributormanagerType::Garbagecollection::Garbagecollection(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    selectiontoremove = ConfigParser::parse<vespalib::string>("selectiontoremove", __lines, "");
    ConfigParser::stripLinesForKey("selectiontoremove", __remainingValuesToParse);
    interval = ConfigParser::parse<int32_t>("interval", __lines, 0);
    ConfigParser::stripLinesForKey("interval", __remainingValuesToParse);
}

bool
InternalStorDistributormanagerType::Garbagecollection::
operator==(const Garbagecollection& __rhs) const noexcept
{
    return (selectiontoremove == __rhs.selectiontoremove &&
            interval == __rhs.interval);
}

bool
InternalStorDistributormanagerType::Garbagecollection::
operator!=(const Garbagecollection& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalStorDistributormanagerType::Garbagecollection::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("selectiontoremove");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(selectiontoremove));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("interval");
        __c.setString("type", "int");
        __c.setLong("value", interval);
    }
}

InternalStorDistributormanagerType::Garbagecollection::Garbagecollection(const vespalib::slime::Inspector & __inspector)
{
    selectiontoremove = __inspector["selectiontoremove"]["value"].asString().make_string();
    interval = __inspector["interval"]["value"].asLong();
}

InternalStorDistributormanagerType::Garbagecollection::Garbagecollection(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    selectiontoremove = ::config::internal::ValueConverter<vespalib::string>()(__inspector["selectiontoremove"], "");
    interval = ::config::internal::ValueConverter<int32_t>()(__inspector["interval"], 0);
}

InternalStorDistributormanagerType::MinimumReplicaCountingMode
InternalStorDistributormanagerType::getMinimumReplicaCountingMode(const vespalib::string& name)
{
    if (name == "TRUSTED") {
        return MinimumReplicaCountingMode::TRUSTED;
    } else if (name == "ANY") {
        return MinimumReplicaCountingMode::ANY;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalStorDistributormanagerType::getMinimumReplicaCountingModeName(MinimumReplicaCountingMode t)
{
    switch (t) {
        case MinimumReplicaCountingMode::TRUSTED: return "TRUSTED";
        case MinimumReplicaCountingMode::ANY: return "ANY";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalStorDistributormanagerType::MinimumReplicaCountingMode InternalStorDistributormanagerType::InternalMinimumReplicaCountingModeConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalStorDistributormanagerType::getMinimumReplicaCountingMode(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalStorDistributormanagerType::MinimumReplicaCountingMode InternalStorDistributormanagerType::InternalMinimumReplicaCountingModeConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalStorDistributormanagerType::getMinimumReplicaCountingMode(__inspector.asString().make_string());
}
InternalStorDistributormanagerType::MinimumReplicaCountingMode InternalStorDistributormanagerType::InternalMinimumReplicaCountingModeConverter::operator()(const ::vespalib::slime::Inspector & __inspector, MinimumReplicaCountingMode __eDefault) {
    if (__inspector.valid()) {
        return InternalStorDistributormanagerType::getMinimumReplicaCountingMode(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalStorDistributormanagerType::InternalStorDistributormanagerType()
    : maxpendingidealstateoperations(100),
      splitsize(16772216),
      splitcount(1024),
      joinsize(16000000),
      joincount(512),
      minsplitcount(16),
      garbagecollection(),
      inlinebucketsplitting(true),
      blockedstatecheckers(),
      enableRevert(true),
      maximumNodesPerMerge(16),
      startDistributorThread(true),
      ticksBeforeWait(10),
      ticksWaitTimeMs(1),
      maxProcessTimeMs(5000),
      priorityMergeMoveToIdealNode(165),
      priorityMergeOutOfSyncCopies(120),
      priorityMergeTooFewCopies(120),
      priorityMergeGlobalBuckets(115),
      priorityActivateNoExistingActive(100),
      priorityActivateWithExistingActive(100),
      priorityDeleteBucketCopy(120),
      priorityJoinBuckets(155),
      prioritySplitDistributionBits(200),
      prioritySplitLargeBucket(175),
      prioritySplitInconsistentBucket(110),
      priorityGarbageCollection(200),
      enableJoinForSiblingLessBuckets(false),
      enableInconsistentJoin(false),
      enableHostInfoReporting(true),
      minimumReplicaCountingMode(MinimumReplicaCountingMode::TRUSTED),
      disableBucketActivation(false),
      maxClusterClockSkewSec(1),
      sequenceMutatingOperations(true),
      inhibitMergeSendingOnBusyNodeDurationSec(1),
      allowStaleReadsDuringClusterStateTransitions(false),
      simulatedDbPruningLatencyMsec(0),
      simulatedDbMergingLatencyMsec(0),
      useBtreeDatabase(true),
      restartWithFastUpdatePathIfAllGetTimestampsAreConsistent(true),
      mergeOperationsDisabled(false),
      useWeakInternalReadConsistencyForClientGets(false),
      enableMetadataOnlyFetchPhaseForInconsistentUpdates(true),
      maxConsecutivelyInhibitedMaintenanceTicks(20),
      prioritizeGlobalBucketMerges(true),
      maxActivationInhibitedOutOfSyncGroups(0),
      numDistributorStripes(0),
      implicitlyClearBucketPriorityOnSchedule(true),
      useUnorderedMergeChaining(true),
      inhibitDefaultMergesWhenGlobalMergesPending(true),
      enableTwoPhaseGarbageCollection(true),
      enableConditionProbing(true),
      enableOperationCancellation(false)
{
}

InternalStorDistributormanagerType::InternalStorDistributormanagerType(const InternalStorDistributormanagerType & __rhs) = default;
InternalStorDistributormanagerType & InternalStorDistributormanagerType::operator =(const InternalStorDistributormanagerType & __rhs) = default;
InternalStorDistributormanagerType::InternalStorDistributormanagerType(InternalStorDistributormanagerType && __rhs) noexcept = default;
InternalStorDistributormanagerType & InternalStorDistributormanagerType::operator =(InternalStorDistributormanagerType && __rhs) noexcept = default;
InternalStorDistributormanagerType::~InternalStorDistributormanagerType() = default; 
InternalStorDistributormanagerType::InternalStorDistributormanagerType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        maxpendingidealstateoperations = ConfigParser::parse<int32_t>("maxpendingidealstateoperations", __lines, 100);
        ConfigParser::stripLinesForKey("maxpendingidealstateoperations", __remainingValuesToParse);
        splitsize = ConfigParser::parse<int32_t>("splitsize", __lines, 16772216);
        ConfigParser::stripLinesForKey("splitsize", __remainingValuesToParse);
        splitcount = ConfigParser::parse<int32_t>("splitcount", __lines, 1024);
        ConfigParser::stripLinesForKey("splitcount", __remainingValuesToParse);
        joinsize = ConfigParser::parse<int32_t>("joinsize", __lines, 16000000);
        ConfigParser::stripLinesForKey("joinsize", __remainingValuesToParse);
        joincount = ConfigParser::parse<int32_t>("joincount", __lines, 512);
        ConfigParser::stripLinesForKey("joincount", __remainingValuesToParse);
        minsplitcount = ConfigParser::parse<int32_t>("minsplitcount", __lines, 16);
        ConfigParser::stripLinesForKey("minsplitcount", __remainingValuesToParse);
        garbagecollection = ConfigParser::parseStruct<Garbagecollection>("garbagecollection", __lines);
        ConfigParser::stripLinesForKey("garbagecollection", __remainingValuesToParse);
        inlinebucketsplitting = ConfigParser::parse<bool>("inlinebucketsplitting", __lines, true);
        ConfigParser::stripLinesForKey("inlinebucketsplitting", __remainingValuesToParse);
        blockedstatecheckers = ConfigParser::parseArray<::config::StringVector>("blockedstatecheckers", __lines);
        ConfigParser::stripLinesForKey("blockedstatecheckers", __remainingValuesToParse);
        enableRevert = ConfigParser::parse<bool>("enable_revert", __lines, true);
        ConfigParser::stripLinesForKey("enable_revert", __remainingValuesToParse);
        maximumNodesPerMerge = ConfigParser::parse<int32_t>("maximum_nodes_per_merge", __lines, 16);
        ConfigParser::stripLinesForKey("maximum_nodes_per_merge", __remainingValuesToParse);
        startDistributorThread = ConfigParser::parse<bool>("start_distributor_thread", __lines, true);
        ConfigParser::stripLinesForKey("start_distributor_thread", __remainingValuesToParse);
        ticksBeforeWait = ConfigParser::parse<int32_t>("ticks_before_wait", __lines, 10);
        ConfigParser::stripLinesForKey("ticks_before_wait", __remainingValuesToParse);
        ticksWaitTimeMs = ConfigParser::parse<int32_t>("ticks_wait_time_ms", __lines, 1);
        ConfigParser::stripLinesForKey("ticks_wait_time_ms", __remainingValuesToParse);
        maxProcessTimeMs = ConfigParser::parse<int32_t>("max_process_time_ms", __lines, 5000);
        ConfigParser::stripLinesForKey("max_process_time_ms", __remainingValuesToParse);
        priorityMergeMoveToIdealNode = ConfigParser::parse<int32_t>("priority_merge_move_to_ideal_node", __lines, 165);
        ConfigParser::stripLinesForKey("priority_merge_move_to_ideal_node", __remainingValuesToParse);
        priorityMergeOutOfSyncCopies = ConfigParser::parse<int32_t>("priority_merge_out_of_sync_copies", __lines, 120);
        ConfigParser::stripLinesForKey("priority_merge_out_of_sync_copies", __remainingValuesToParse);
        priorityMergeTooFewCopies = ConfigParser::parse<int32_t>("priority_merge_too_few_copies", __lines, 120);
        ConfigParser::stripLinesForKey("priority_merge_too_few_copies", __remainingValuesToParse);
        priorityMergeGlobalBuckets = ConfigParser::parse<int32_t>("priority_merge_global_buckets", __lines, 115);
        ConfigParser::stripLinesForKey("priority_merge_global_buckets", __remainingValuesToParse);
        priorityActivateNoExistingActive = ConfigParser::parse<int32_t>("priority_activate_no_existing_active", __lines, 100);
        ConfigParser::stripLinesForKey("priority_activate_no_existing_active", __remainingValuesToParse);
        priorityActivateWithExistingActive = ConfigParser::parse<int32_t>("priority_activate_with_existing_active", __lines, 100);
        ConfigParser::stripLinesForKey("priority_activate_with_existing_active", __remainingValuesToParse);
        priorityDeleteBucketCopy = ConfigParser::parse<int32_t>("priority_delete_bucket_copy", __lines, 120);
        ConfigParser::stripLinesForKey("priority_delete_bucket_copy", __remainingValuesToParse);
        priorityJoinBuckets = ConfigParser::parse<int32_t>("priority_join_buckets", __lines, 155);
        ConfigParser::stripLinesForKey("priority_join_buckets", __remainingValuesToParse);
        prioritySplitDistributionBits = ConfigParser::parse<int32_t>("priority_split_distribution_bits", __lines, 200);
        ConfigParser::stripLinesForKey("priority_split_distribution_bits", __remainingValuesToParse);
        prioritySplitLargeBucket = ConfigParser::parse<int32_t>("priority_split_large_bucket", __lines, 175);
        ConfigParser::stripLinesForKey("priority_split_large_bucket", __remainingValuesToParse);
        prioritySplitInconsistentBucket = ConfigParser::parse<int32_t>("priority_split_inconsistent_bucket", __lines, 110);
        ConfigParser::stripLinesForKey("priority_split_inconsistent_bucket", __remainingValuesToParse);
        priorityGarbageCollection = ConfigParser::parse<int32_t>("priority_garbage_collection", __lines, 200);
        ConfigParser::stripLinesForKey("priority_garbage_collection", __remainingValuesToParse);
        enableJoinForSiblingLessBuckets = ConfigParser::parse<bool>("enable_join_for_sibling_less_buckets", __lines, false);
        ConfigParser::stripLinesForKey("enable_join_for_sibling_less_buckets", __remainingValuesToParse);
        enableInconsistentJoin = ConfigParser::parse<bool>("enable_inconsistent_join", __lines, false);
        ConfigParser::stripLinesForKey("enable_inconsistent_join", __remainingValuesToParse);
        enableHostInfoReporting = ConfigParser::parse<bool>("enable_host_info_reporting", __lines, true);
        ConfigParser::stripLinesForKey("enable_host_info_reporting", __remainingValuesToParse);
        minimumReplicaCountingMode = getMinimumReplicaCountingMode(ConfigParser::parse<vespalib::string>("minimum_replica_counting_mode", __lines, "TRUSTED"));
        ConfigParser::stripLinesForKey("minimum_replica_counting_mode", __remainingValuesToParse);
        disableBucketActivation = ConfigParser::parse<bool>("disable_bucket_activation", __lines, false);
        ConfigParser::stripLinesForKey("disable_bucket_activation", __remainingValuesToParse);
        maxClusterClockSkewSec = ConfigParser::parse<int32_t>("max_cluster_clock_skew_sec", __lines, 1);
        ConfigParser::stripLinesForKey("max_cluster_clock_skew_sec", __remainingValuesToParse);
        sequenceMutatingOperations = ConfigParser::parse<bool>("sequence_mutating_operations", __lines, true);
        ConfigParser::stripLinesForKey("sequence_mutating_operations", __remainingValuesToParse);
        inhibitMergeSendingOnBusyNodeDurationSec = ConfigParser::parse<int32_t>("inhibit_merge_sending_on_busy_node_duration_sec", __lines, 1);
        ConfigParser::stripLinesForKey("inhibit_merge_sending_on_busy_node_duration_sec", __remainingValuesToParse);
        allowStaleReadsDuringClusterStateTransitions = ConfigParser::parse<bool>("allow_stale_reads_during_cluster_state_transitions", __lines, false);
        ConfigParser::stripLinesForKey("allow_stale_reads_during_cluster_state_transitions", __remainingValuesToParse);
        simulatedDbPruningLatencyMsec = ConfigParser::parse<int32_t>("simulated_db_pruning_latency_msec", __lines, 0);
        ConfigParser::stripLinesForKey("simulated_db_pruning_latency_msec", __remainingValuesToParse);
        simulatedDbMergingLatencyMsec = ConfigParser::parse<int32_t>("simulated_db_merging_latency_msec", __lines, 0);
        ConfigParser::stripLinesForKey("simulated_db_merging_latency_msec", __remainingValuesToParse);
        useBtreeDatabase = ConfigParser::parse<bool>("use_btree_database", __lines, true);
        ConfigParser::stripLinesForKey("use_btree_database", __remainingValuesToParse);
        restartWithFastUpdatePathIfAllGetTimestampsAreConsistent = ConfigParser::parse<bool>("restart_with_fast_update_path_if_all_get_timestamps_are_consistent", __lines, true);
        ConfigParser::stripLinesForKey("restart_with_fast_update_path_if_all_get_timestamps_are_consistent", __remainingValuesToParse);
        mergeOperationsDisabled = ConfigParser::parse<bool>("merge_operations_disabled", __lines, false);
        ConfigParser::stripLinesForKey("merge_operations_disabled", __remainingValuesToParse);
        useWeakInternalReadConsistencyForClientGets = ConfigParser::parse<bool>("use_weak_internal_read_consistency_for_client_gets", __lines, false);
        ConfigParser::stripLinesForKey("use_weak_internal_read_consistency_for_client_gets", __remainingValuesToParse);
        enableMetadataOnlyFetchPhaseForInconsistentUpdates = ConfigParser::parse<bool>("enable_metadata_only_fetch_phase_for_inconsistent_updates", __lines, true);
        ConfigParser::stripLinesForKey("enable_metadata_only_fetch_phase_for_inconsistent_updates", __remainingValuesToParse);
        maxConsecutivelyInhibitedMaintenanceTicks = ConfigParser::parse<int32_t>("max_consecutively_inhibited_maintenance_ticks", __lines, 20);
        ConfigParser::stripLinesForKey("max_consecutively_inhibited_maintenance_ticks", __remainingValuesToParse);
        prioritizeGlobalBucketMerges = ConfigParser::parse<bool>("prioritize_global_bucket_merges", __lines, true);
        ConfigParser::stripLinesForKey("prioritize_global_bucket_merges", __remainingValuesToParse);
        maxActivationInhibitedOutOfSyncGroups = ConfigParser::parse<int32_t>("max_activation_inhibited_out_of_sync_groups", __lines, 0);
        ConfigParser::stripLinesForKey("max_activation_inhibited_out_of_sync_groups", __remainingValuesToParse);
        numDistributorStripes = ConfigParser::parse<int32_t>("num_distributor_stripes", __lines, 0);
        ConfigParser::stripLinesForKey("num_distributor_stripes", __remainingValuesToParse);
        implicitlyClearBucketPriorityOnSchedule = ConfigParser::parse<bool>("implicitly_clear_bucket_priority_on_schedule", __lines, true);
        ConfigParser::stripLinesForKey("implicitly_clear_bucket_priority_on_schedule", __remainingValuesToParse);
        useUnorderedMergeChaining = ConfigParser::parse<bool>("use_unordered_merge_chaining", __lines, true);
        ConfigParser::stripLinesForKey("use_unordered_merge_chaining", __remainingValuesToParse);
        inhibitDefaultMergesWhenGlobalMergesPending = ConfigParser::parse<bool>("inhibit_default_merges_when_global_merges_pending", __lines, true);
        ConfigParser::stripLinesForKey("inhibit_default_merges_when_global_merges_pending", __remainingValuesToParse);
        enableTwoPhaseGarbageCollection = ConfigParser::parse<bool>("enable_two_phase_garbage_collection", __lines, true);
        ConfigParser::stripLinesForKey("enable_two_phase_garbage_collection", __remainingValuesToParse);
        enableConditionProbing = ConfigParser::parse<bool>("enable_condition_probing", __lines, true);
        ConfigParser::stripLinesForKey("enable_condition_probing", __remainingValuesToParse);
        enableOperationCancellation = ConfigParser::parse<bool>("enable_operation_cancellation", __lines, false);
        ConfigParser::stripLinesForKey("enable_operation_cancellation", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalStorDistributormanagerType::
operator==(const InternalStorDistributormanagerType& __rhs) const noexcept
{
    return (maxpendingidealstateoperations == __rhs.maxpendingidealstateoperations &&
            splitsize == __rhs.splitsize &&
            splitcount == __rhs.splitcount &&
            joinsize == __rhs.joinsize &&
            joincount == __rhs.joincount &&
            minsplitcount == __rhs.minsplitcount &&
            garbagecollection == __rhs.garbagecollection &&
            inlinebucketsplitting == __rhs.inlinebucketsplitting &&
            blockedstatecheckers == __rhs.blockedstatecheckers &&
            enableRevert == __rhs.enableRevert &&
            maximumNodesPerMerge == __rhs.maximumNodesPerMerge &&
            startDistributorThread == __rhs.startDistributorThread &&
            ticksBeforeWait == __rhs.ticksBeforeWait &&
            ticksWaitTimeMs == __rhs.ticksWaitTimeMs &&
            maxProcessTimeMs == __rhs.maxProcessTimeMs &&
            priorityMergeMoveToIdealNode == __rhs.priorityMergeMoveToIdealNode &&
            priorityMergeOutOfSyncCopies == __rhs.priorityMergeOutOfSyncCopies &&
            priorityMergeTooFewCopies == __rhs.priorityMergeTooFewCopies &&
            priorityMergeGlobalBuckets == __rhs.priorityMergeGlobalBuckets &&
            priorityActivateNoExistingActive == __rhs.priorityActivateNoExistingActive &&
            priorityActivateWithExistingActive == __rhs.priorityActivateWithExistingActive &&
            priorityDeleteBucketCopy == __rhs.priorityDeleteBucketCopy &&
            priorityJoinBuckets == __rhs.priorityJoinBuckets &&
            prioritySplitDistributionBits == __rhs.prioritySplitDistributionBits &&
            prioritySplitLargeBucket == __rhs.prioritySplitLargeBucket &&
            prioritySplitInconsistentBucket == __rhs.prioritySplitInconsistentBucket &&
            priorityGarbageCollection == __rhs.priorityGarbageCollection &&
            enableJoinForSiblingLessBuckets == __rhs.enableJoinForSiblingLessBuckets &&
            enableInconsistentJoin == __rhs.enableInconsistentJoin &&
            enableHostInfoReporting == __rhs.enableHostInfoReporting &&
            minimumReplicaCountingMode == __rhs.minimumReplicaCountingMode &&
            disableBucketActivation == __rhs.disableBucketActivation &&
            maxClusterClockSkewSec == __rhs.maxClusterClockSkewSec &&
            sequenceMutatingOperations == __rhs.sequenceMutatingOperations &&
            inhibitMergeSendingOnBusyNodeDurationSec == __rhs.inhibitMergeSendingOnBusyNodeDurationSec &&
            allowStaleReadsDuringClusterStateTransitions == __rhs.allowStaleReadsDuringClusterStateTransitions &&
            simulatedDbPruningLatencyMsec == __rhs.simulatedDbPruningLatencyMsec &&
            simulatedDbMergingLatencyMsec == __rhs.simulatedDbMergingLatencyMsec &&
            useBtreeDatabase == __rhs.useBtreeDatabase &&
            restartWithFastUpdatePathIfAllGetTimestampsAreConsistent == __rhs.restartWithFastUpdatePathIfAllGetTimestampsAreConsistent &&
            mergeOperationsDisabled == __rhs.mergeOperationsDisabled &&
            useWeakInternalReadConsistencyForClientGets == __rhs.useWeakInternalReadConsistencyForClientGets &&
            enableMetadataOnlyFetchPhaseForInconsistentUpdates == __rhs.enableMetadataOnlyFetchPhaseForInconsistentUpdates &&
            maxConsecutivelyInhibitedMaintenanceTicks == __rhs.maxConsecutivelyInhibitedMaintenanceTicks &&
            prioritizeGlobalBucketMerges == __rhs.prioritizeGlobalBucketMerges &&
            maxActivationInhibitedOutOfSyncGroups == __rhs.maxActivationInhibitedOutOfSyncGroups &&
            numDistributorStripes == __rhs.numDistributorStripes &&
            implicitlyClearBucketPriorityOnSchedule == __rhs.implicitlyClearBucketPriorityOnSchedule &&
            useUnorderedMergeChaining == __rhs.useUnorderedMergeChaining &&
            inhibitDefaultMergesWhenGlobalMergesPending == __rhs.inhibitDefaultMergesWhenGlobalMergesPending &&
            enableTwoPhaseGarbageCollection == __rhs.enableTwoPhaseGarbageCollection &&
            enableConditionProbing == __rhs.enableConditionProbing &&
            enableOperationCancellation == __rhs.enableOperationCancellation);
}

bool
InternalStorDistributormanagerType::
operator!=(const InternalStorDistributormanagerType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalStorDistributormanagerType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxpendingidealstateoperations");
        __c.setString("type", "int");
        __c.setLong("value", maxpendingidealstateoperations);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("splitsize");
        __c.setString("type", "int");
        __c.setLong("value", splitsize);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("splitcount");
        __c.setString("type", "int");
        __c.setLong("value", splitcount);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("joinsize");
        __c.setString("type", "int");
        __c.setLong("value", joinsize);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("joincount");
        __c.setString("type", "int");
        __c.setLong("value", joincount);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("minsplitcount");
        __c.setString("type", "int");
        __c.setLong("value", minsplitcount);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("garbagecollection");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        garbagecollection.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("inlinebucketsplitting");
        __c.setString("type", "bool");
        __c.setBool("value", inlinebucketsplitting);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("blockedstatecheckers");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : blockedstatecheckers) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "string");
            __c3.setString("value", vespalib::Memory(child));
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("enable_revert");
        __c.setString("type", "bool");
        __c.setBool("value", enableRevert);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maximum_nodes_per_merge");
        __c.setString("type", "int");
        __c.setLong("value", maximumNodesPerMerge);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("start_distributor_thread");
        __c.setString("type", "bool");
        __c.setBool("value", startDistributorThread);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("ticks_before_wait");
        __c.setString("type", "int");
        __c.setLong("value", ticksBeforeWait);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("ticks_wait_time_ms");
        __c.setString("type", "int");
        __c.setLong("value", ticksWaitTimeMs);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("max_process_time_ms");
        __c.setString("type", "int");
        __c.setLong("value", maxProcessTimeMs);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("priority_merge_move_to_ideal_node");
        __c.setString("type", "int");
        __c.setLong("value", priorityMergeMoveToIdealNode);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("priority_merge_out_of_sync_copies");
        __c.setString("type", "int");
        __c.setLong("value", priorityMergeOutOfSyncCopies);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("priority_merge_too_few_copies");
        __c.setString("type", "int");
        __c.setLong("value", priorityMergeTooFewCopies);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("priority_merge_global_buckets");
        __c.setString("type", "int");
        __c.setLong("value", priorityMergeGlobalBuckets);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("priority_activate_no_existing_active");
        __c.setString("type", "int");
        __c.setLong("value", priorityActivateNoExistingActive);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("priority_activate_with_existing_active");
        __c.setString("type", "int");
        __c.setLong("value", priorityActivateWithExistingActive);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("priority_delete_bucket_copy");
        __c.setString("type", "int");
        __c.setLong("value", priorityDeleteBucketCopy);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("priority_join_buckets");
        __c.setString("type", "int");
        __c.setLong("value", priorityJoinBuckets);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("priority_split_distribution_bits");
        __c.setString("type", "int");
        __c.setLong("value", prioritySplitDistributionBits);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("priority_split_large_bucket");
        __c.setString("type", "int");
        __c.setLong("value", prioritySplitLargeBucket);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("priority_split_inconsistent_bucket");
        __c.setString("type", "int");
        __c.setLong("value", prioritySplitInconsistentBucket);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("priority_garbage_collection");
        __c.setString("type", "int");
        __c.setLong("value", priorityGarbageCollection);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("enable_join_for_sibling_less_buckets");
        __c.setString("type", "bool");
        __c.setBool("value", enableJoinForSiblingLessBuckets);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("enable_inconsistent_join");
        __c.setString("type", "bool");
        __c.setBool("value", enableInconsistentJoin);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("enable_host_info_reporting");
        __c.setString("type", "bool");
        __c.setBool("value", enableHostInfoReporting);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("minimum_replica_counting_mode");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getMinimumReplicaCountingModeName(minimumReplicaCountingMode)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("disable_bucket_activation");
        __c.setString("type", "bool");
        __c.setBool("value", disableBucketActivation);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("max_cluster_clock_skew_sec");
        __c.setString("type", "int");
        __c.setLong("value", maxClusterClockSkewSec);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("sequence_mutating_operations");
        __c.setString("type", "bool");
        __c.setBool("value", sequenceMutatingOperations);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("inhibit_merge_sending_on_busy_node_duration_sec");
        __c.setString("type", "int");
        __c.setLong("value", inhibitMergeSendingOnBusyNodeDurationSec);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("allow_stale_reads_during_cluster_state_transitions");
        __c.setString("type", "bool");
        __c.setBool("value", allowStaleReadsDuringClusterStateTransitions);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("simulated_db_pruning_latency_msec");
        __c.setString("type", "int");
        __c.setLong("value", simulatedDbPruningLatencyMsec);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("simulated_db_merging_latency_msec");
        __c.setString("type", "int");
        __c.setLong("value", simulatedDbMergingLatencyMsec);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("use_btree_database");
        __c.setString("type", "bool");
        __c.setBool("value", useBtreeDatabase);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("restart_with_fast_update_path_if_all_get_timestamps_are_consistent");
        __c.setString("type", "bool");
        __c.setBool("value", restartWithFastUpdatePathIfAllGetTimestampsAreConsistent);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("merge_operations_disabled");
        __c.setString("type", "bool");
        __c.setBool("value", mergeOperationsDisabled);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("use_weak_internal_read_consistency_for_client_gets");
        __c.setString("type", "bool");
        __c.setBool("value", useWeakInternalReadConsistencyForClientGets);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("enable_metadata_only_fetch_phase_for_inconsistent_updates");
        __c.setString("type", "bool");
        __c.setBool("value", enableMetadataOnlyFetchPhaseForInconsistentUpdates);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("max_consecutively_inhibited_maintenance_ticks");
        __c.setString("type", "int");
        __c.setLong("value", maxConsecutivelyInhibitedMaintenanceTicks);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("prioritize_global_bucket_merges");
        __c.setString("type", "bool");
        __c.setBool("value", prioritizeGlobalBucketMerges);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("max_activation_inhibited_out_of_sync_groups");
        __c.setString("type", "int");
        __c.setLong("value", maxActivationInhibitedOutOfSyncGroups);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("num_distributor_stripes");
        __c.setString("type", "int");
        __c.setLong("value", numDistributorStripes);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("implicitly_clear_bucket_priority_on_schedule");
        __c.setString("type", "bool");
        __c.setBool("value", implicitlyClearBucketPriorityOnSchedule);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("use_unordered_merge_chaining");
        __c.setString("type", "bool");
        __c.setBool("value", useUnorderedMergeChaining);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("inhibit_default_merges_when_global_merges_pending");
        __c.setString("type", "bool");
        __c.setBool("value", inhibitDefaultMergesWhenGlobalMergesPending);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("enable_two_phase_garbage_collection");
        __c.setString("type", "bool");
        __c.setBool("value", enableTwoPhaseGarbageCollection);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("enable_condition_probing");
        __c.setString("type", "bool");
        __c.setBool("value", enableConditionProbing);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("enable_operation_cancellation");
        __c.setString("type", "bool");
        __c.setBool("value", enableOperationCancellation);
    }
}

InternalStorDistributormanagerType::InternalStorDistributormanagerType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    maxpendingidealstateoperations = __inspector["maxpendingidealstateoperations"]["value"].asLong();
    splitsize = __inspector["splitsize"]["value"].asLong();
    splitcount = __inspector["splitcount"]["value"].asLong();
    joinsize = __inspector["joinsize"]["value"].asLong();
    joincount = __inspector["joincount"]["value"].asLong();
    minsplitcount = __inspector["minsplitcount"]["value"].asLong();
    garbagecollection = Garbagecollection(__inspector["garbagecollection"]["value"]);
    inlinebucketsplitting = __inspector["inlinebucketsplitting"]["value"].asBool();
    for (size_t __i = 0; __i < __inspector["blockedstatecheckers"]["value"].children(); __i++) {
        blockedstatecheckers.push_back(__inspector["blockedstatecheckers"]["value"][__i]["value"].asString().make_string());
    }
    enableRevert = __inspector["enable_revert"]["value"].asBool();
    maximumNodesPerMerge = __inspector["maximum_nodes_per_merge"]["value"].asLong();
    startDistributorThread = __inspector["start_distributor_thread"]["value"].asBool();
    ticksBeforeWait = __inspector["ticks_before_wait"]["value"].asLong();
    ticksWaitTimeMs = __inspector["ticks_wait_time_ms"]["value"].asLong();
    maxProcessTimeMs = __inspector["max_process_time_ms"]["value"].asLong();
    priorityMergeMoveToIdealNode = __inspector["priority_merge_move_to_ideal_node"]["value"].asLong();
    priorityMergeOutOfSyncCopies = __inspector["priority_merge_out_of_sync_copies"]["value"].asLong();
    priorityMergeTooFewCopies = __inspector["priority_merge_too_few_copies"]["value"].asLong();
    priorityMergeGlobalBuckets = __inspector["priority_merge_global_buckets"]["value"].asLong();
    priorityActivateNoExistingActive = __inspector["priority_activate_no_existing_active"]["value"].asLong();
    priorityActivateWithExistingActive = __inspector["priority_activate_with_existing_active"]["value"].asLong();
    priorityDeleteBucketCopy = __inspector["priority_delete_bucket_copy"]["value"].asLong();
    priorityJoinBuckets = __inspector["priority_join_buckets"]["value"].asLong();
    prioritySplitDistributionBits = __inspector["priority_split_distribution_bits"]["value"].asLong();
    prioritySplitLargeBucket = __inspector["priority_split_large_bucket"]["value"].asLong();
    prioritySplitInconsistentBucket = __inspector["priority_split_inconsistent_bucket"]["value"].asLong();
    priorityGarbageCollection = __inspector["priority_garbage_collection"]["value"].asLong();
    enableJoinForSiblingLessBuckets = __inspector["enable_join_for_sibling_less_buckets"]["value"].asBool();
    enableInconsistentJoin = __inspector["enable_inconsistent_join"]["value"].asBool();
    enableHostInfoReporting = __inspector["enable_host_info_reporting"]["value"].asBool();
    minimumReplicaCountingMode = getMinimumReplicaCountingMode(__inspector["minimum_replica_counting_mode"]["value"].asString().make_string());
    disableBucketActivation = __inspector["disable_bucket_activation"]["value"].asBool();
    maxClusterClockSkewSec = __inspector["max_cluster_clock_skew_sec"]["value"].asLong();
    sequenceMutatingOperations = __inspector["sequence_mutating_operations"]["value"].asBool();
    inhibitMergeSendingOnBusyNodeDurationSec = __inspector["inhibit_merge_sending_on_busy_node_duration_sec"]["value"].asLong();
    allowStaleReadsDuringClusterStateTransitions = __inspector["allow_stale_reads_during_cluster_state_transitions"]["value"].asBool();
    simulatedDbPruningLatencyMsec = __inspector["simulated_db_pruning_latency_msec"]["value"].asLong();
    simulatedDbMergingLatencyMsec = __inspector["simulated_db_merging_latency_msec"]["value"].asLong();
    useBtreeDatabase = __inspector["use_btree_database"]["value"].asBool();
    restartWithFastUpdatePathIfAllGetTimestampsAreConsistent = __inspector["restart_with_fast_update_path_if_all_get_timestamps_are_consistent"]["value"].asBool();
    mergeOperationsDisabled = __inspector["merge_operations_disabled"]["value"].asBool();
    useWeakInternalReadConsistencyForClientGets = __inspector["use_weak_internal_read_consistency_for_client_gets"]["value"].asBool();
    enableMetadataOnlyFetchPhaseForInconsistentUpdates = __inspector["enable_metadata_only_fetch_phase_for_inconsistent_updates"]["value"].asBool();
    maxConsecutivelyInhibitedMaintenanceTicks = __inspector["max_consecutively_inhibited_maintenance_ticks"]["value"].asLong();
    prioritizeGlobalBucketMerges = __inspector["prioritize_global_bucket_merges"]["value"].asBool();
    maxActivationInhibitedOutOfSyncGroups = __inspector["max_activation_inhibited_out_of_sync_groups"]["value"].asLong();
    numDistributorStripes = __inspector["num_distributor_stripes"]["value"].asLong();
    implicitlyClearBucketPriorityOnSchedule = __inspector["implicitly_clear_bucket_priority_on_schedule"]["value"].asBool();
    useUnorderedMergeChaining = __inspector["use_unordered_merge_chaining"]["value"].asBool();
    inhibitDefaultMergesWhenGlobalMergesPending = __inspector["inhibit_default_merges_when_global_merges_pending"]["value"].asBool();
    enableTwoPhaseGarbageCollection = __inspector["enable_two_phase_garbage_collection"]["value"].asBool();
    enableConditionProbing = __inspector["enable_condition_probing"]["value"].asBool();
    enableOperationCancellation = __inspector["enable_operation_cancellation"]["value"].asBool();
}

InternalStorDistributormanagerType::InternalStorDistributormanagerType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    maxpendingidealstateoperations = ::config::internal::ValueConverter<int32_t>()(__inspector["maxpendingidealstateoperations"], 100);
    splitsize = ::config::internal::ValueConverter<int32_t>()(__inspector["splitsize"], 16772216);
    splitcount = ::config::internal::ValueConverter<int32_t>()(__inspector["splitcount"], 1024);
    joinsize = ::config::internal::ValueConverter<int32_t>()(__inspector["joinsize"], 16000000);
    joincount = ::config::internal::ValueConverter<int32_t>()(__inspector["joincount"], 512);
    minsplitcount = ::config::internal::ValueConverter<int32_t>()(__inspector["minsplitcount"], 16);
    garbagecollection = ::config::internal::ValueConverter<Garbagecollection>()(__inspector["garbagecollection"]);
    inlinebucketsplitting = ::config::internal::ValueConverter<bool>()(__inspector["inlinebucketsplitting"], true);
    ::config::internal::VectorInserter<::config::StringVector> __blockedstatecheckersInserter(blockedstatecheckers);
    __inspector["blockedstatecheckers"].traverse(__blockedstatecheckersInserter);
    enableRevert = ::config::internal::ValueConverter<bool>()(__inspector["enable_revert"], true);
    maximumNodesPerMerge = ::config::internal::ValueConverter<int32_t>()(__inspector["maximum_nodes_per_merge"], 16);
    startDistributorThread = ::config::internal::ValueConverter<bool>()(__inspector["start_distributor_thread"], true);
    ticksBeforeWait = ::config::internal::ValueConverter<int32_t>()(__inspector["ticks_before_wait"], 10);
    ticksWaitTimeMs = ::config::internal::ValueConverter<int32_t>()(__inspector["ticks_wait_time_ms"], 1);
    maxProcessTimeMs = ::config::internal::ValueConverter<int32_t>()(__inspector["max_process_time_ms"], 5000);
    priorityMergeMoveToIdealNode = ::config::internal::ValueConverter<int32_t>()(__inspector["priority_merge_move_to_ideal_node"], 165);
    priorityMergeOutOfSyncCopies = ::config::internal::ValueConverter<int32_t>()(__inspector["priority_merge_out_of_sync_copies"], 120);
    priorityMergeTooFewCopies = ::config::internal::ValueConverter<int32_t>()(__inspector["priority_merge_too_few_copies"], 120);
    priorityMergeGlobalBuckets = ::config::internal::ValueConverter<int32_t>()(__inspector["priority_merge_global_buckets"], 115);
    priorityActivateNoExistingActive = ::config::internal::ValueConverter<int32_t>()(__inspector["priority_activate_no_existing_active"], 100);
    priorityActivateWithExistingActive = ::config::internal::ValueConverter<int32_t>()(__inspector["priority_activate_with_existing_active"], 100);
    priorityDeleteBucketCopy = ::config::internal::ValueConverter<int32_t>()(__inspector["priority_delete_bucket_copy"], 120);
    priorityJoinBuckets = ::config::internal::ValueConverter<int32_t>()(__inspector["priority_join_buckets"], 155);
    prioritySplitDistributionBits = ::config::internal::ValueConverter<int32_t>()(__inspector["priority_split_distribution_bits"], 200);
    prioritySplitLargeBucket = ::config::internal::ValueConverter<int32_t>()(__inspector["priority_split_large_bucket"], 175);
    prioritySplitInconsistentBucket = ::config::internal::ValueConverter<int32_t>()(__inspector["priority_split_inconsistent_bucket"], 110);
    priorityGarbageCollection = ::config::internal::ValueConverter<int32_t>()(__inspector["priority_garbage_collection"], 200);
    enableJoinForSiblingLessBuckets = ::config::internal::ValueConverter<bool>()(__inspector["enable_join_for_sibling_less_buckets"], false);
    enableInconsistentJoin = ::config::internal::ValueConverter<bool>()(__inspector["enable_inconsistent_join"], false);
    enableHostInfoReporting = ::config::internal::ValueConverter<bool>()(__inspector["enable_host_info_reporting"], true);
    minimumReplicaCountingMode = InternalMinimumReplicaCountingModeConverter()(__inspector["minimum_replica_counting_mode"], MinimumReplicaCountingMode::TRUSTED);
    disableBucketActivation = ::config::internal::ValueConverter<bool>()(__inspector["disable_bucket_activation"], false);
    maxClusterClockSkewSec = ::config::internal::ValueConverter<int32_t>()(__inspector["max_cluster_clock_skew_sec"], 1);
    sequenceMutatingOperations = ::config::internal::ValueConverter<bool>()(__inspector["sequence_mutating_operations"], true);
    inhibitMergeSendingOnBusyNodeDurationSec = ::config::internal::ValueConverter<int32_t>()(__inspector["inhibit_merge_sending_on_busy_node_duration_sec"], 1);
    allowStaleReadsDuringClusterStateTransitions = ::config::internal::ValueConverter<bool>()(__inspector["allow_stale_reads_during_cluster_state_transitions"], false);
    simulatedDbPruningLatencyMsec = ::config::internal::ValueConverter<int32_t>()(__inspector["simulated_db_pruning_latency_msec"], 0);
    simulatedDbMergingLatencyMsec = ::config::internal::ValueConverter<int32_t>()(__inspector["simulated_db_merging_latency_msec"], 0);
    useBtreeDatabase = ::config::internal::ValueConverter<bool>()(__inspector["use_btree_database"], true);
    restartWithFastUpdatePathIfAllGetTimestampsAreConsistent = ::config::internal::ValueConverter<bool>()(__inspector["restart_with_fast_update_path_if_all_get_timestamps_are_consistent"], true);
    mergeOperationsDisabled = ::config::internal::ValueConverter<bool>()(__inspector["merge_operations_disabled"], false);
    useWeakInternalReadConsistencyForClientGets = ::config::internal::ValueConverter<bool>()(__inspector["use_weak_internal_read_consistency_for_client_gets"], false);
    enableMetadataOnlyFetchPhaseForInconsistentUpdates = ::config::internal::ValueConverter<bool>()(__inspector["enable_metadata_only_fetch_phase_for_inconsistent_updates"], true);
    maxConsecutivelyInhibitedMaintenanceTicks = ::config::internal::ValueConverter<int32_t>()(__inspector["max_consecutively_inhibited_maintenance_ticks"], 20);
    prioritizeGlobalBucketMerges = ::config::internal::ValueConverter<bool>()(__inspector["prioritize_global_bucket_merges"], true);
    maxActivationInhibitedOutOfSyncGroups = ::config::internal::ValueConverter<int32_t>()(__inspector["max_activation_inhibited_out_of_sync_groups"], 0);
    numDistributorStripes = ::config::internal::ValueConverter<int32_t>()(__inspector["num_distributor_stripes"], 0);
    implicitlyClearBucketPriorityOnSchedule = ::config::internal::ValueConverter<bool>()(__inspector["implicitly_clear_bucket_priority_on_schedule"], true);
    useUnorderedMergeChaining = ::config::internal::ValueConverter<bool>()(__inspector["use_unordered_merge_chaining"], true);
    inhibitDefaultMergesWhenGlobalMergesPending = ::config::internal::ValueConverter<bool>()(__inspector["inhibit_default_merges_when_global_merges_pending"], true);
    enableTwoPhaseGarbageCollection = ::config::internal::ValueConverter<bool>()(__inspector["enable_two_phase_garbage_collection"], true);
    enableConditionProbing = ::config::internal::ValueConverter<bool>()(__inspector["enable_condition_probing"], true);
    enableOperationCancellation = ::config::internal::ValueConverter<bool>()(__inspector["enable_operation_cancellation"], false);
}

} // namespace internal

} // namespace vespa::config::content::core
